/*
 *	boxview.h
 *
 *	The Box View is used to hold a set of views, which will be surrounded
 *	by a frame and a title (which may both be disabled).
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_BOXVIEW_H_
#define	_OPaC_BOXVIEW_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

/****************************************************************/

#ifndef _OPaC_CONTROL_H_
#include <OPaC/control.h>
#endif

#ifndef _OPaC_XTWIDGET_H_
#include <OPaC/xtwidget.h>
#endif

#ifndef	_OPaC_LOOKCOLOR_H_
#include <OPaC/lookcolor.h>
#endif

/****************************************************************/

/*
 *	Box View class. This can be used to associate elements
 *	into logical groups.
 */

class OP_PUBLIC OPBoxView : public OPControl, public OPxTitleWidget
{
	
protected:
	OPTextTagRef		title;				//	title of the box
	Bool				enable_title;		//	TRUE => draw the title
	Bool				enable_frame;		//	TRUE => draw the frame
	OPLookColor			back_color;			//	color of background or OP_COLOR_NONE
	OPLookColor			frame_color;		//	color of frame or OP_COLOR_NONE
	
protected:
	OPBoxView ();
	virtual ~ OPBoxView ();
	
	virtual void StoreInstance (OPStorageIn storage);		/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);		/* OVERRIDDEN */
	
	virtual void SetupTextTag ();
	virtual void UpdateTextTag ();
	virtual void ChangeGeometry (OPRectIn);					/* OVERRIDDEN */
	
	virtual OPView* ReturnObjectAsView ();
	
public:
	virtual void Delete ();									/* OVERRIDDEN */
	
	virtual Bool SetEnableTitle (Bool yes);
	virtual Bool SetEnableFrame (Bool yes);
	virtual Bool ReturnEnableTitle () const;
	virtual Bool ReturnEnableFrame () const;
	
	virtual Bool SetFont (OPGrailFontIn font);
	virtual Bool SetTitle (OPStringIn text);
	virtual Bool SetBackColor (OPLookColor color);
	virtual Bool SetFrameColor (OPLookColor color);
	
	virtual OPGrailFontRef ReturnFont () const;
	virtual OPStringRef ReturnTitle () const;
	
	virtual OPLookColor ReturnBackColor () const;
	virtual OPLookColor ReturnFrameColor () const;
	
	//	Inherited methods :
	
	virtual void GetMinSize (OPCoordOut, OPCoordOut) const;	/* OVERRIDDEN */
	virtual void GetMaxSize (OPCoordOut, OPCoordOut) const;	/* OVERRIDDEN */
	
	virtual void Draw (OPGrailPortIn, OPRectIn, OPDrawOp = OP_DRAW_ALL);/* OVERRIDDEN */
	
	OP_DEFINE_SLOT (OPTextTag, title, SetTitle, ReturnTitle);
	OP_DEFINE_SLOT (Bool, enable_title, SetEnableTitle, ReturnEnableTitle);
	OP_DEFINE_SLOT (Bool, enable_frame, SetEnableFrame, ReturnEnableFrame);
	OP_DEFINE_SLOT (OPLookColor, back_color, SetBackColor, ReturnBackColor);	
	OP_DEFINE_SLOT (OPLookColor, frame_color, SetFrameColor, ReturnFrameColor);
	
	OP_DEFINE_CLASS (OPBoxView, OPControl);
};

#endif /* _OPaC_BOXVIEW_H_ */
