/*
 *	colorpixel.h
 *
 *	Defines the "color pixel" representation, which is an automatically
 *	converting pixel representation format.
 *
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_COLORPIXEL_H_
#define	_OPaC_COLORPIXEL_H_

#include <OPaC/object.h>

#ifndef _OPaC_LOOKCOLOR_H_
#include <OPaC/lookcolor.h>
#endif

/****************************************************************/

/*
 *	The ColorPixel class represents colored pixels as RGB
 *	values and/or as internal "look" colors.
 */

class OP_PUBLIC OPColorPixel
{
	friend class OPStorageRef;
	friend class OPLookMan;
	
private:
	mutable Card32		pixel;				//	real (grail) pixel value
	OPLookColor			color;				//	look manager color (if any)
	
protected:
	void StoreInstance (OPStorageIn) const;
	void RestoreInstance (OPStorageIn);
	
	void UpdatePixel () const;
	
public:
	OPColorPixel ();
	~OPColorPixel ();
	OPColorPixel (OPValueIn value);
	
	OPColorPixel (Card32);
	OPColorPixel (OPLookColor);
	OPColorPixel (OPColorPixelIn);
	OPColorPixel (OPAtomIn);
	
	OPColorPixel (Real, Real, Real, Real = 0.0);
	OPColorPixel (Card8, Card8, Card8, Card8 = 0);
	
	OPColorPixel& operator = (OPValueIn);
	OPColorPixel& operator = (Card32);
	OPColorPixel& operator = (OPLookColor);
	OPColorPixel& operator = (OPColorPixelIn);
	OPColorPixel& operator = (OPAtomIn);
	
	Bool operator == (Card32) const;
	Bool operator == (OPLookColor) const;
	Bool operator == (OPColorPixelIn) const;
	Bool operator == (OPAtomIn) const;
	
	friend OP_PUBLIC Bool operator == (Card32, OPColorPixelIn);
	friend OP_PUBLIC Bool operator == (OPLookColor, OPColorPixelIn);
	friend OP_PUBLIC Bool operator == (OPAtomIn, OPColorPixelIn);
	
	Bool operator != (Card32) const;
	Bool operator != (OPLookColor) const;
	Bool operator != (OPColorPixelIn) const;
	Bool operator != (OPAtomIn) const;
	
	friend OP_PUBLIC Bool operator != (Card32, OPColorPixelIn);
	friend OP_PUBLIC Bool operator != (OPLookColor, OPColorPixelIn);
	friend OP_PUBLIC Bool operator != (OPAtomIn, OPColorPixelIn);
	
	operator Card32 () const;
	operator OPLookColor () const;
	operator OPAtomRef () const;
	
	void SetRed (Card8);
	void SetGreen (Card8);
	void SetBlue (Card8);
	void SetTransparency (Card8);
	void SetLuminance (Card8);
	
	void SetRed (Real);
	void SetGreen (Real);
	void SetBlue (Real);
	void SetTransparency (Real);
	void SetLuminance (Real);
	
	Card8 Red () const;
	Card8 Green () const;
	Card8 Blue () const;
	Card8 Transparency () const;
	Card8 Luminance () const;
	
	Real RealRed () const;
	Real RealGreen () const;
	Real RealBlue () const;
	Real RealTransparency () const;
	Real RealLuminance () const;
	
	static const OPColorPixel none;
	static Card32 MakeCompactRGB (Card8 r, Card8 g, Card8 b, Card8 t = 0);
	static Card32 MakeCompactRGB (Real r, Real g, Real b, Real t = 0);
};

/****************************************************************/

#endif /* _OPaC_COLORPIXEL_H_ */
