/*
 *	colslider.h
 *
 *	Colored Slider class definition.
 *	
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_COLSLIDER_H_
#define	_OPaC_COLSLIDER_H_

#ifndef _OPaC_SLIDER_H_
#include <OPaC/slider.h>
#endif

#ifndef _OPaC_COLORPIXEL_H_
#include <OPaC/colorpixel.h>
#endif

/****************************************************************/

/*
 *	Colored Slider class.
 */

class OP_PUBLIC OPColSlider : public OPSlider
{
	
protected:
	OPColorPixel		min_color;
	OPColorPixel		max_color;
	
protected:
	OPColSlider ();
	virtual ~ OPColSlider ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
	virtual void DrawBody (OPGrailPortIn port);			/* OVERRIDDEN */
	
public:
	virtual Bool SetMinColor (OPColorPixelIn color);
	virtual Bool SetMaxColor (OPColorPixelIn color);
	
	virtual OPColorPixel ReturnMinColor () const;
	virtual OPColorPixel ReturnMaxColor () const;
	virtual OPColorPixel ReturnCurrentColor () const;
	
	OP_DEFINE_SLOT (OPColorPixel, min_color, SetMinColor, ReturnMinColor);
	OP_DEFINE_SLOT (OPColorPixel, max_color, SetMaxColor, ReturnMaxColor);
	OP_DEFINE_SLOT_READ (OPColorPixel, current_color, ReturnCurrentColor);
	
	OP_DEFINE_CLASS (OPColSlider, OPSlider);
};

/****************************************************************/

#endif /* _OPaC_COLSLIDER_H_ */
