/*
 *	datalog.h
 *
 *	Data logger : this is a simple way to store lots of data
 *	(which may be stored on disk when needed). The data can
 *	be of the same type or variable.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_DATALOG_H_
#define	_OPaC_DATALOG_H_

#ifndef _OPaC_CONTAINER_H_
#include <OPaC/container.h>
#endif

#ifndef _OPaC_XTSMARTSRC_H_
#include <OPaC/xtsmartsrc.h>
#endif

/****************************************************************/

/*
 *	The Data Logging class stores information either as
 *	a circular list or as an ever growing list.
 */

class OP_PUBLIC OPDataLog : public OPGenericContainer, public OPxSmartSource
{
	
	union OPaC_DataLogUnion {
		Card32*			card_array;
		Int32*			int_array;
		Bool*			bool_array;
		Real*			real_array;
		Byte*			byte_array;
	};
	
protected:
	OPSimpleType		type;				//	type of values (if simple)
	OPaC_DataLogUnion	simple;				//	array of simple values
	Card32				simple_size;		//	size of a single simple value
	OPValueArray		complex_array;		//	array of complex values
	
	Count				index;				//	current # of elements stored
	Count				array_size;			//	size of storage array
	Count				cyclic_size;		//	size of cyclic buffer
	
	Bool				is_cyclic;			//	TRUE => cyclic data logger
	Bool				is_simple;			//	TRUE => simple data type (not an object)
	
protected:
	OPDataLog ();
	virtual ~ OPDataLog ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
	void GrowSimpleArray (Card32 num);
	void ComputeSimpleSize ();
	Bool InsertAnyType (void*);
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual void InitLog (OPSimpleType type, Count cyclic_size = 0);
	virtual OPSimpleType ReturnType () const;
	
	virtual Bool InsertValue (OPValueIn value);
	virtual Bool GetNthValue (Count index, OPValueOut value) const;
	
	virtual void AutoInsertFromSource ();
	virtual void FlushCacheAndAutoInsertFromSource ();
	
	virtual Bool InsertCard32 (Card32 x);
	virtual Bool InsertInt32 (Int32 x);
	virtual Bool InsertBool (Bool x);
	virtual Bool InsertReal (Real x);
	
	virtual Bool GetNthCard32 (Count index, Card32Out x) const;
	virtual Bool GetNthInt32 (Count index, Int32Out x) const;
	virtual Bool GetNthBool (Count index, BoolOut x) const;
	virtual Bool GetNthReal (Count index, RealOut x) const;
	
	virtual Bool GetMultipleCard32 (Count index, Card32* x, Count num) const;
	virtual Bool GetMultipleInt32 (Count index, Int32* x, Count num) const;
	virtual Bool GetMultipleBool (Count index, Bool* x, Count num) const;
	virtual Bool GetMultipleReal (Count index, Real* x, Count num) const;
	
	//	Other overridden methods :
	
	virtual Count TotalCount () const;					/* OVERRIDDEN */
	virtual Bool Get (Count index, OPObjectOut object);	/* OVERRIDDEN */
	virtual Bool SpecifyExtraction (OPArrayIn x);		/* OVERRIDDEN */
	
	OP_DEFINE_CLASS (OPDataLog, OPGenericContainer);
};

/****************************************************************/

#endif /* _OPaC_DATALOG_H_ */

