/*
 *	dbscrlist.h
 *
 *	Database-based scroll list. This provides the mechanisms
 *	needed to easily represent a collection of items visually.
 *
 *	(C) Copyright 1997-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_DBSCRLIST_H_
#define _OPaC_DBSCRLIST_H_

#include <OPaC/view.h>
#include <OPaC/icostyle.h>

/****************************************************************/

/*
 *	This class builds a scroll list from the elements
 *	described in the data base.
 */

class OP_PUBLIC OPDBScrollList : public OPView
{
	static OPAtomRef	atom_index;
	static OPAtomRef	atom_name;
	static OPAtomRef	atom_data;
	static OPAtomRef	atom_text;
	static OPAtomRef	atom_large_icon;
	static OPAtomRef	atom_small_icon;
	static OPAtomRef	atom_mark_icon;
	
protected:
	OPDataBaseRef		db;						//	data base
	OPEntityModelRef	db_model;				//	model for an entity
	OPDBProviderRef		db_provider;			//	provider used to extract from the data base
	OPHashTableRef		db_hash;				//	hash table for uniqueness
	
	OPObjectPaneRef		pane;					//	pane used to represent the data
	OPStringRef			pane_title;				//	title above the pane
	OPTextCaptionRef	caption;				//	the caption displays the title
	OPSelGridRef		grid;					//	the grid is provided by the pane
	
	Bool				is_unique;				//	TRUE => remove duplicates
	
protected:
	OPDBScrollList ();
	virtual ~ OPDBScrollList ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
	virtual Bool GetSelectedEntity (OPEntityOut entity, Count nth) const;
	
public:
	virtual Bool SetupDataBase (OPStringIn title, Bool unique = TRUE);
	virtual Bool Insert (OPObjectIn index, OPTextIn name, OPObjectIn data, OPTextIn text, OPLookIcon = OP_ICON_NONE, OPLookIcon = OP_ICON_NONE, OPLookIcon = OP_ICON_NONE);
	
	virtual Bool Update ();
	virtual Bool RemoveAll ();
	virtual Bool HandleCellActivated ();
	
	OPObjectPaneRet	 ReturnPane () const		{ return this->pane; }
	OPStringRet		 ReturnTitle () const		{ return this->pane_title; }
	OPTextCaptionRet ReturnCaption () const		{ return this->caption; }
	OPSelGridRet     ReturnGrid () const		{ return this->grid; }
	
	virtual Bool SetSelectedCell (Count index, Bool report = TRUE);
	virtual Bool GetSelectedCell (CountOut index, Count nth = 0) const;
	
	virtual Count CountSelectedItems () const;
	
	virtual Bool GetSelectedIndex (OPObjectOut index, Count nth = 0) const;
	virtual Bool GetSelectedName (OPTextOut name, Count nth = 0) const;
	virtual Bool GetSelectedData (OPObjectOut data, Count nth = 0) const;
	virtual Bool GetSelectedText (OPTextOut text, Count nth = 0) const;
	virtual Bool GetSelectedLargeIcon (OPLookIconOut icon, Count nth = 0) const;
	virtual Bool GetSelectedSmallIcon (OPLookIconOut icon, Count nth = 0) const;
	virtual Bool GetSelectedMarkIcon (OPLookIconOut icon, Count nth = 0) const;
	
	OP_DEFINE_CLASS (OPDBScrollList, OPView);
};

/****************************************************************/

#endif /* _OPaC_DBSCRLIST_H_ */
