/*
 *	dockview.h
 *
 *	The docking view can be torn off its parent view and moved
 *	around in its own window. It provides the basic support for
 *	palettes and main menus.
 *
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_DOCKVIEW_H_
#define	_OPaC_DOCKVIEW_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

#ifndef _OPaC_VIEW_H_
#include <OPaC/view.h>
#endif

/****************************************************************/

OP_TYPEDEF (Card32, OPDockStyle);

#define	OP_DOCK_HANDLE_NONE		((OPDockStyle)(0x00000000))	//	no handle
#define	OP_DOCK_HANDLE_LEFT		((OPDockStyle)(0x00000001))	//	left handle (vertical)
#define	OP_DOCK_HANDLE_TOP		((OPDockStyle)(0x00000002))	//	right handle (horizontal)

#define	OP_DOCK_STRIPE_SINGLE	((OPDockStyle)(0x00000000))
#define	OP_DOCK_STRIPE_DOUBLE	((OPDockStyle)(0x00000010))
#define	OP_DOCK_STRIPE_THIN		((OPDockStyle)(0x00000020))
#define	OP_DOCK_STRIPE_BUMP		((OPDockStyle)(0x00000030))

#define	OP_DOCK_SEL_HANDLE		((OPDockStyle)(0x00000100))
#define	OP_DOCK_SEL_BORDER		((OPDockStyle)(0x00000200))
#define	OP_DOCK_SEL_BODY		((OPDockStyle)(0x00000400))

#define	OP_DOCK_DRAW_GLOW		((OPDockStyle)(0x00010000))
#define	OP_DOCK_DRAW_SHADE		((OPDockStyle)(0x00020000))
#define	OP_DOCK_DRAW_FRAME		((OPDockStyle)(0x00040000))

#define	OP_DOCK_HANDLE_MASK		((OPDockStyle)(0x0000000F))
#define	OP_DOCK_STRIPE_MASK		((OPDockStyle)(0x000000F0))
#define	OP_DOCK_SEL_MASK		((OPDockStyle)(0x00000F00))
#define	OP_DOCK_DRAW_MASK		((OPDockStyle)(0x000F0000))

/****************************************************************/

/*
 *	
 */

class OP_PUBLIC OPDockView : public OPView
{
	
protected:
	Bool				is_dragging;			//	TRUE => currently dragging
	
	OPDockStyle			style;					//	docking style
	
	OPWindowRef			dock_window;			//	window owned by the undocked view
	OPCoord				hot_x;					//	[x] of original click position
	OPCoord				hot_y;					//	[y] of original click position
	OPCoord				border;					//	width of sensitive border
	OPCoord				width;					//	width of handle
	
protected:
	OPDockView ();
	virtual ~ OPDockView ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
	virtual void UpdateHandleWidth ();
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual Bool StartDragging ();
	
	virtual Bool SetStyle (OPDockStyleIn style);
	virtual OPDockStyle ReturnStyle () const;
	
	virtual void ButtonPress (OPEventIn event);			/* OVERRIDDEN */
	virtual void ButtonRelease (OPEventIn event);		/* OVERRIDDEN */
	virtual void DragNotify (OPEventIn event);			/* OVERRIDDEN */
	virtual void Draw (OPGrailPortIn, OPRectIn, OPDrawOp = OP_DRAW_ALL);/* OVERRIDDEN */
	
	OP_DEFINE_SLOT (OPDockStyle, style, SetStyle, ReturnStyle);
	
	OP_DEFINE_CLASS (OPDockView, OPView);
};

#endif /* _OPaC_DOCKVIEW_H_ */

