/*
 *	image.h
 *
 *	The Image class displays an image (useful for backgrounds
 *	and large icons).
 *
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_IMAGE_H_
#define	_OPaC_IMAGE_H_

#ifndef _OPaC_VIEW_H_
#include <OPaC/view.h>
#endif

/****************************************************************/

/*
 *	Image class.
 */

class OP_PUBLIC OPImage : public OPView
{
	
private:
	OPGrailPixmapRef	pixmap;					//	associated pixmap
	OPStringRef			name;					//	name of source file
	
	Bool				is_centered;			//	TRUE => image is centered in bounding box
	Bool				is_tiled;				//	TRUE => image is tiled
	
protected:
	OPImage ();
	virtual ~ OPImage ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual Bool Initialise (OPStringIn name, Bool center = TRUE, Bool tile = FALSE);
	virtual Bool SetDefaultSize ();
	
	virtual Bool SetImageName (OPStringIn name);
	virtual Bool SetCenteredImage (Bool yes);
	virtual Bool SetTiledImage (Bool yes);
	
	virtual OPStringRet ReturnImageName () const;
	virtual Bool GetImageSize (OPCoordOut dx, OPCoordOut dy) const;
	
	virtual Bool IsImageCentered () const;
	virtual Bool IsImageTiled () const;
	
	//	Inherited methods :
	
	virtual void Awake ();												/* OVERRIDDEN */
	virtual void Draw (OPGrailPortIn, OPRectIn, OPDrawOp = OP_DRAW_ALL);/* OVERRIDDEN */
	
	OP_DEFINE_SLOT (Bool, is_centered, SetCenteredImage, IsImageCentered);
	OP_DEFINE_SLOT (Bool, is_tiled, SetTiledImage, IsImageTiled);
	OP_DEFINE_SLOT (OPString, image_name, SetImageName, ReturnImageName);
	
	OP_DEFINE_CLASS (OPImage, OPView);
};

/****************************************************************/

#endif /* _OPaC_IMAGE_H_ */
