/*
 *	pipestream.h
 *
 *	The piped stream class provides interprocess and remote
 *	communication (with other OPaC compatible apps).
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_PIPESTREAM_H_
#define	_OPaC_PIPESTREAM_H_

#ifndef _OPaC_STREAM_H_
#include <OPaC/stream.h>
#endif

#ifndef _OPaC_STRING_H_
#include <OPaC/string.h>
#endif

/****************************************************************/

/*
 *	
 */

class OP_PUBLIC OPPipeClient : public OPStream
{
	
protected:
	struct OPaC_CPipe*	pipe;
	
	OPStringRef			host_name;
	OPStringRef			server_name;
	OPStringRef			pipe_name;
	Real				timeout;
	
protected:
	OPPipeClient ();
	virtual ~ OPPipeClient ();
	
	virtual void StoreInstance (OPStorageIn storage);
	virtual void RestoreInstance (OPStorageIn storage);
	
public:
	virtual Bool SetHostName (OPStringIn name);
	virtual Bool SetServerName (OPStringIn name);
	virtual Bool SetPipeName (OPStringIn name);
	virtual Bool SetTimeOut (Real time);
	
	//	Abstract communication:
	
	virtual Bool Request (const void* request, Count request_len,
						  void* answer, Count& answer_len);
	
	//	Lower level communication:
	
	virtual void Open ();
	virtual Count Read (void*, Count);
	virtual Count Write (const void*, Count);
	virtual Bool Sync ();
	virtual void Close ();
	
	OP_DEFINE_CLASS (OPPipeClient, OPStream);
};

/****************************************************************/

/*
 *
 */

class OP_PUBLIC OPPipeServer : public OPStream
{
	
protected:
	struct OPaC_SPipe*	pipe;
	
	OPStringRef			server_name;
	OPStringRef			pipe_name;
	Real				timeout;
	
protected:
	OPPipeServer ();
	virtual ~ OPPipeServer ();
	
	virtual void StoreInstance (OPStorageIn storage);
	virtual void RestoreInstance (OPStorageIn storage);
	
public:
	virtual Bool SetServerName (OPStringIn name);
	virtual Bool SetPipeName (OPStringIn name);
	virtual Bool SetTimeOut (Real time);
	
	//	Lower level communication:
	
	virtual void Create ();
	virtual void Open ();
	virtual Count Read (void*, Count);
	virtual Count Write (const void*, Count);
	virtual Bool Sync ();
	virtual void Close ();
	virtual void Kill ();
	
	OP_DEFINE_CLASS (OPPipeServer, OPStream);
};

/****************************************************************/

/*
 *	Exception class associated to the stream operations. The
 *	stream methods may throw such exceptions.
 */

class OP_PUBLIC OPExcPipe : public OPException
{

public:
	enum Error {
		CANNOT_CONNECT,						//	could not connect
		TIMEOUT								//	time-out
	};
	
protected:
	Error				error;				// error code
	
public:
	OPExcPipe (Error e) : error (e) { }
	virtual Error GetError () const { return error; }
};

#endif /* _OPaC_PIPESTREAM_H_ */
