/*
 *	scrgrid.h
 *
 *	The Scroll Grid implements a support for a large cell grid
 *	including the addition of scrollers.
 *
 *	(C) Copyright 1997-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_SCROLLGRID_H_
#define	_OPaC_SCROLLGRID_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

#ifndef _OPaC_CONTROL_H_
#include <OPaC/control.h>
#endif

/****************************************************************/

/*
 *	The scroll grid class provides a cell grid with zero,
 *	one or two scrollers.
 */

class OP_PUBLIC OPScrollGrid : public OPControl
{
	
protected:
	OPScrollViewRef		scroll_view;
	OPScrollerRef		h_scroller;
	OPScrollerRef		v_scroller;
	OPCellGridRef		grid;
	
	Bool				is_valid;
	Bool				permanent_h_scroller;
	Bool				permanent_v_scroller;
	
protected:
	OPScrollGrid ();
	virtual ~ OPScrollGrid ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
	virtual void ChangeGeometry (OPRectIn box);			/* OVERRIDDEN */
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual void Initialise (OPCellGridIn grid, OPRectIn box);
	virtual void Draw (OPGrailPortIn, OPRectIn, OPDrawOp); /* OVERRIDDEN */
	
	virtual Bool SetPermanentVScroller (Bool yes);
	virtual Bool SetPermanentHScroller (Bool yes);
	virtual Bool ReturnPermanentVScroller () const;
	virtual Bool ReturnPermanentHScroller () const;
	
	virtual Bool SetCellGrid (OPCellGridIn grid);
	virtual Bool GetCellGrid (OPCellGridOut grid) const;
	virtual OPCellGridRet ReturnCellGrid () const;
	
	virtual void GridChangedSize ();
	
	OP_DEFINE_SLOT (OPCellGrid, cell_grid, SetCellGrid, ReturnCellGrid);
	OP_DEFINE_SLOT (Bool, permanent_v_scroller, SetPermanentVScroller, ReturnPermanentVScroller);
	OP_DEFINE_SLOT (Bool, permanent_h_scroller, SetPermanentHScroller, ReturnPermanentHScroller);
	OP_DEFINE_CLASS (OPScrollGrid, OPControl);
};

/****************************************************************/

#endif /* _OPaC_SCROLLGRID_H_ */
