/*
 *	scroller.h
 *
 *	Scroller class definition. The scroller class is used to represent what other
 *	call scroll bars...
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_SCROLLER_H_
#define	_OPaC_SCROLLER_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

#ifndef _OPaC_CONTROL_H_
#include <OPaC/control.h>
#endif

#ifndef _OPaC_SCRSTYLE_H_
#include <OPaC/scrstyle.h>
#endif

/****************************************************************/

/*
 *	Scroller class.
 */

class OP_PUBLIC OPScroller : public OPControl
{
	
protected:
	static Bool			look_inited;			//	TRUE => look has been initialised :
	static OPCoord		look_breadth;			//	- scroller breadth (width or height)
	static OPCoord		look_frame;				//	- frame size around widgets
	static OPCoord		look_3D_effect;			//	- size of 3D effect
	static Count		look_repeat;			//	- automatic repeat increment
	static Count		look_start;				//	- automatic repeat start time
	
protected:
	Real				value;					//	0.0 to 1.0, depending on knob position
	Real				ratio;					//	0.0 to 1.0, visible / total size ratio
	Real				increment;				//	increment to be used when scrolling
	
	OPCoord				knob_min;				//	minimum pos where knob can move
	OPCoord				knob_max;				//	maximum pos where knob can move
	
	OPCoord				knob_pos;				//	knob position (offset) in scroll bar
	OPCoord				knob_size;				//	knob size (in points)
	OPRelCoord			knob_click;				//	click offset between knob and cursor
	
	Count				tick_count;				//	0 if no automatic scrolling occurred
	
	Bool				is_horiz;				//	TRUE => scroller is horizontal
	Bool				is_knob_ok;				//	TRUE => knob is ok and will be drawn
	Bool				is_ok;					//	TRUE => ok, has been fully initialised
	
	OPScrollArrowPos	init_arrow_pos;			//	selected arrow position(s)
	OPScrollArrowPos	real_arrow_pos;			//	real position(s) depending on available size
	
	OPIconRef			arrow_incr;				//	incrementing arrow (down or right)
	OPIconRef			arrow_decr;				//	decrementing arrow (up or left)
	
protected:
	OPScroller ();
	virtual ~ OPScroller ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
	virtual void ChangeGeometry (OPRectIn);				/* OVERRIDDEN */
	
	virtual void InitArrow (OPIconInOut arrow, Card32 id);
	virtual void InitIcons (Bool is_horiz);
	virtual void CheckSpaceForParts ();
	virtual void UpdateKnobSize ();
	
	virtual void DrawKnob (OPGrailPortIn port);
	virtual void DrawBody (OPGrailPortIn port);
	virtual void Scroll (Card32 id);
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual void Awake ();								/* OVERRIDDEN */
	virtual void AwakeFromApp (OPAppIn app);			/* OVERRIDDEN */
	virtual void AwakeFromObject (OPObjectIn object);	/* OVERRIDDEN */
	
	virtual void Initialise (OPCoord x, OPCoord y, OPCoord size, Bool is_horiz);
	virtual void SetArrowPos (OPScrollArrowPos pos);
	
	virtual Bool SetIncrement (Real);
	virtual Bool SetVisibleValue (Real);
	virtual Bool SetVisibleRatio (Real);
	
	virtual Real ReturnIncrement () const;
	virtual Real ReturnVisibleValue () const;
	virtual Real ReturnVisibleRatio () const;
	
	virtual OPCoord ReturnBreadth () const;
	
	//	Observers exported for run-time binding :
	
	virtual void OnClick (OPObjectIn src, Card32 id);
	virtual void OnContClick (OPObjectIn src, Card32 id);
	
	//	Inherited methods :
	
	virtual void GetMinSize (OPCoordOut dx, OPCoordOut dy) const;/* OVERRIDDEN */
	virtual void GetMaxSize (OPCoordOut dx, OPCoordOut dy) const;/* OVERRIDDEN */
	
	virtual void Broadcast (OPAtomIn, OPObjectIn);						/* OVERRIDDEN */
	virtual void Draw (OPGrailPortIn, OPRectIn, OPDrawOp = OP_DRAW_ALL);/* OVERRIDDEN */
	
//	virtual void KeyDown (OPEventIn event);					/* OVERRIDDEN */
//	virtual void KeyUp (OPEventIn event);					/* OVERRIDDEN */
	
	virtual void ButtonPress (OPEventIn event);				/* OVERRIDDEN */
	virtual void ButtonRelease (OPEventIn event);			/* OVERRIDDEN */
	virtual void DragNotify (OPEventIn event);				/* OVERRIDDEN */
	
//	virtual void EnterNotify (OPEventIn event);				/* OVERRIDDEN */
//	virtual void LeaveNotify (OPEventIn event);				/* OVERRIDDEN */
	
	OP_DEFINE_SLOT (Real, visible_value, SetVisibleValue, ReturnVisibleValue);
	OP_DEFINE_SLOT (Real, visible_ratio, SetVisibleRatio, ReturnVisibleRatio);
	OP_DEFINE_SLOT (Real, increment, SetIncrement, ReturnIncrement);
	OP_DEFINE_CLASS (OPScroller, OPControl);
};

/****************************************************************/

#endif /* _OPaC_SCROLLER_H_ */
