/*
 *	spring.h
 *
 *	A spring maps a stretching factor with a size and can be used to
 *	dynamically resize and move objects using a spring board.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef	_OPaC_SPRING_H_
#define	_OPaC_SPRING_H_

#ifndef _OPaC_OPEXPORT_H_
#include <OPaC/opexport.h>
#endif

/****************************************************************/

#define	OP_SPRING_MIN	((Real)(0.0))			//	shrink to virtually nothing
#define	OP_SPRING_ONE	((Real)(1.0))			//	normal size
#define	OP_SPRING_MAX	((Real)(1000.0))		//	stretch to (near) infinity

/****************************************************************/

/*
 *	The spring curve
 */

class OP_PUBLIC OPSpring : public OPObject
{
	struct OPSpringSample
	{
		OPSpringSample*	next;					//	next sample in list
		OPSpringSample*	prev;					//	previous sample in list
		
		Real			stretch;				//	stretching factor [0..n]
		OPCoord			size;					//	corresponding size
	};

protected:
	OPSpringSample*		head;					//	head of spring sample list
	OPSpringSample*		tail;					//	tail of spring sample list

	Count				count;					//	current # of samples
	OPSpringBandRef		band;					//	band associated with this spring
	OPCoord				size;					//	current default size
	
protected:
	OPSpring ();
	virtual ~ OPSpring ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	
public:
	virtual void Delete ();								/* OVERRIDDEN */
	
	virtual void Initialise (OPSpringBandIn band, OPCoord size = 0, Real slope = 0);
	
	virtual void Reset (OPCoord size);
	virtual void Reset (OPCoord size, Real slope);
	virtual void Insert (Real stretch, OPCoord size);
	virtual void Remove (Real stretch);
	virtual void RemoveAll ();
	
	virtual void Merge (OPSpringIn spring);
	
	virtual OPCoord ComputeDefaultSize (Real stretch);
	virtual OPCoord ReturnDefaultSize () const;
	
	virtual OPCoord FindSize (Real stretch) const;
	virtual Real FindStretch (OPCoord size) const;
	virtual Real FindSlope (Real stretch) const;

	virtual Count CountSamples () const;
	virtual Bool GetNthSample (Count index, RealOut, OPCoordOut);

	OP_DEFINE_CLASS (OPSpring, OPObject);
};

/****************************************************************/

#endif /* _OPaC_SPRING_H_ */
