/*
 *	values.h
 *
 *	Encapsulate values (numbers, integers, booleans, reals, etc.)
 *	so that they can be handled as standard OPaC objects.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _OPaC_VALUES_H_
#define	_OPaC_VALUES_H_

#ifndef _OPaC_OBJECT_H_
#include <OPaC/object.h>
#include <OPaC/opexport.h>
#endif

#ifndef _OPaC_COLORPIXEL_H_
#include <OPaC/colorpixel.h>
#endif

#ifndef _OPaC_RECT_H_
#include <OPaC/rect.h>
#endif

/****************************************************************/

OP_DEFINE_SPECIAL_REF_BEGIN (OPValue, OPObject);
	
//	OPValueRef (OPObjectIn x);
//	OPValueRef (OPValueIn x);
//	OPValueRef (OPValue* x);
	OPValueRef (Card32 x);
	OPValueRef (Int32 x);
	OPValueRef (Card64 x);
	OPValueRef (Int64 x);
	OPValueRef (Bool x);
	OPValueRef (Real x);
	OPValueRef (OPColorPixelIn x);
	OPValueRef (OPRectIn x);
	
//	OPValueRef& operator = (OPObjectIn x);
//	OPValueRef& operator = (OPValueIn x);
//	OPValueRef& operator = (OPValue* x);
	OPValueRef& operator = (Card32 x);
	OPValueRef& operator = (Int32 x);
	OPValueRef& operator = (Card64 x);
	OPValueRef& operator = (Int64 x);
	OPValueRef& operator = (Bool x);
	OPValueRef& operator = (Real x);
	OPValueRef& operator = (OPColorPixelIn x);
	OPValueRef& operator = (OPRectIn x);

	operator Card32 () const;
	operator Int32 () const;
	operator Card64 () const;
	operator Int64 () const;
	operator Bool () const;
	operator Real () const;
	operator OPColorPixel () const;
	operator OPRect () const;
	
	OPValue* NewValue ();
	
	friend Bool operator == (OPValueIn v, Card32 x);
	friend Bool operator == (OPValueIn v, Int32 x);
	friend Bool operator == (OPValueIn v, Card64 x);
	friend Bool operator == (OPValueIn v, Int64 x);
	friend Bool operator == (OPValueIn v, Bool x);
	friend Bool operator == (OPValueIn v, Real x);
	friend Bool operator == (OPValueIn v, OPColorPixelIn x);
	friend Bool operator == (OPValueIn v, OPRectIn x);
	
	friend Bool operator == (Card32 x, OPValueIn v);
	friend Bool operator == (Int32 x, OPValueIn v);
	friend Bool operator == (Card64 x, OPValueIn v);
	friend Bool operator == (Int64 x, OPValueIn v);
	friend Bool operator == (Bool x, OPValueIn v);
	friend Bool operator == (Real x, OPValueIn v);
	friend Bool operator == (OPColorPixelIn x, OPValueIn v);
	friend Bool operator == (OPRectIn x, OPValueIn v);
	
	friend Bool operator != (OPValueIn v, Card32 x);
	friend Bool operator != (OPValueIn v, Int32 x);
	friend Bool operator != (OPValueIn v, Card64 x);
	friend Bool operator != (OPValueIn v, Int64 x);
	friend Bool operator != (OPValueIn v, Bool x);
	friend Bool operator != (OPValueIn v, Real x);
	friend Bool operator != (OPValueIn v, OPColorPixelIn x);
	friend Bool operator != (OPValueIn v, OPRectIn x);
	
	friend Bool operator != (Card32 x, OPValueIn v);
	friend Bool operator != (Int32 x, OPValueIn v);
	friend Bool operator != (Card64 x, OPValueIn v);
	friend Bool operator != (Int64 x, OPValueIn v);
	friend Bool operator != (Bool x, OPValueIn v);
	friend Bool operator != (Real x, OPValueIn v);
	friend Bool operator != (OPColorPixelIn x, OPValueIn v);
	friend Bool operator != (OPRectIn x, OPValueIn v);
	
OP_DEFINE_SPECIAL_REF_END (OPValue, OPObject);

/****************************************************************/

extern inline Bool operator == (Card32 x, OPValueIn v) { return (v == x) ? TRUE : FALSE; }
extern inline Bool operator == (Int32 x,  OPValueIn v) { return (v == x) ? TRUE : FALSE; }
extern inline Bool operator == (Card64 x, OPValueIn v) { return (v == x) ? TRUE : FALSE; }
extern inline Bool operator == (Int64 x,  OPValueIn v) { return (v == x) ? TRUE : FALSE; }
extern inline Bool operator == (Bool x,   OPValueIn v) { return (v == x) ? TRUE : FALSE; }
extern inline Bool operator == (Real x,   OPValueIn v) { return (v == x) ? TRUE : FALSE; }
extern inline Bool operator == (OPColorPixelIn x, OPValueIn v) { return (v == x) ? TRUE : FALSE; }
extern inline Bool operator == (OPRectIn x, OPValueIn v) { return (v == x) ? TRUE : FALSE; }

extern inline Bool operator != (Card32 x, OPValueIn v) { return (v == x) ? FALSE : TRUE; }
extern inline Bool operator != (Int32 x,  OPValueIn v) { return (v == x) ? FALSE : TRUE; }
extern inline Bool operator != (Card64 x, OPValueIn v) { return (v == x) ? FALSE : TRUE; }
extern inline Bool operator != (Int64 x,  OPValueIn v) { return (v == x) ? FALSE : TRUE; }
extern inline Bool operator != (Bool x,   OPValueIn v) { return (v == x) ? FALSE : TRUE; }
extern inline Bool operator != (Real x,   OPValueIn v) { return (v == x) ? FALSE : TRUE; }
extern inline Bool operator != (OPValueIn v, Card32 x) { return (v == x) ? FALSE : TRUE; }
extern inline Bool operator != (OPValueIn v, Int32 x)  { return (v == x) ? FALSE : TRUE; }
extern inline Bool operator != (OPValueIn v, Bool x)   { return (v == x) ? FALSE : TRUE; }
extern inline Bool operator != (OPValueIn v, Real x)   { return (v == x) ? FALSE : TRUE; }

extern inline Bool operator != (OPColorPixelIn x, OPValueIn v) { return (v == x) ? FALSE : TRUE; }
extern inline Bool operator != (OPValueIn v, OPColorPixelIn x) { return (v == x) ? FALSE : TRUE; }

extern inline Bool operator != (OPRectIn x, OPValueIn v) { return (v == x) ? FALSE : TRUE; }
extern inline Bool operator != (OPValueIn v, OPRectIn x) { return (v == x) ? FALSE : TRUE; }

/****************************************************************/

/*
 *	Values wrapper class...
 */

class OP_PUBLIC OPValue : public OPObject
{
	
	union OPaC_ValueUnion {
		Card64				the_card64;
		Int64				the_int64;
		Bool				the_bool;
		Real				the_real;
	};
	
protected:
	mutable OPSimpleType	type;			//	type of value
	mutable OPaC_ValueUnion	value;			//	value (depending on the type)
	mutable OPObjectRef		object;			//	object value (if any)
	mutable OPColorPixel	color;			//	color value (if any)
	mutable OPRect			rect;			//	rectangle (if any)
	OPStringRef				auto_format;	//	automatic conversion format
	
protected:
	OPValue ();
	virtual ~OPValue ();
	
	virtual void StoreInstance (OPStorageIn storage);	/* OVERRIDDEN */
	virtual void RestoreInstance (OPStorageIn storage);	/* OVERRIDDEN */

	virtual void RefreshValue () const;
	virtual void ChangedValue ();
	
	virtual Real ToReal () const;
	
public:
	virtual Count ComputeHashValue (Count num) const;	/* OVERRIDDEN */
	virtual int CompareWith (OPObjectIn object) const;	/* OVERRIDDEN */
	
	virtual Bool SetCard32 (Card32 x);
	virtual Bool SetInt32 (Int32 x);
	virtual Bool SetCard64 (Card64 x);
	virtual Bool SetInt64 (Int64 x);
	virtual Bool SetBool (Bool x);
	virtual Bool SetReal (Real x);
	virtual Bool SetObject (OPObjectIn x);
	virtual Bool SetText (OPTextIn x);
	virtual Bool SetArray (OPArrayIn x);
	virtual Bool SetColor (OPColorPixelIn x);
	virtual Bool SetRect (OPRectIn x);
	
	virtual Card32 ReturnCard32 () const;
	virtual Int32  ReturnInt32 () const;
	virtual Card64 ReturnCard64 () const;
	virtual Int64  ReturnInt64 () const;
	virtual Bool   ReturnBool () const;
	virtual Real   ReturnReal () const;
	virtual OPObjectRet ReturnObject () const;
	virtual OPObjectRet ReturnText () const;
	virtual OPObjectRet ReturnArray () const;
	virtual OPColorPixelRet ReturnColor () const;
	virtual OPRectRet ReturnRect () const;

	virtual Bool SetAutoFormat (OPStringIn format);
	virtual OPStringRet ReturnAutoFormat () const;
	
	virtual Bool ConvertToString (OPStringOut text, OPStringIn format = OPStringNull) const;
	virtual Bool ConvertToAtom (OPAtomOut atom, OPStringIn format = OPStringNull) const;
	
	virtual int CompareWith (Card32 x) const;
	virtual int CompareWith (Int32 x) const;
	virtual int CompareWith (Card64 x) const;
	virtual int CompareWith (Int64 x) const;
	virtual int CompareWith (Bool x) const;
	virtual int CompareWith (Real x) const;
	virtual int CompareWith (OPColorPixelIn x) const;
	virtual int CompareWith (OPRectIn x) const;
	
	virtual OPSimpleType ReturnType () const;
	virtual Bool IsObject () const;
	
	OP_DEFINE_SLOT (Card32,   card_32, SetCard32, ReturnCard32);
	OP_DEFINE_SLOT (Int32,    int_32,  SetInt32,  ReturnInt32);
	OP_DEFINE_SLOT (Card64,   card_64, SetCard64, ReturnCard64);
	OP_DEFINE_SLOT (Int64,    int_64,  SetInt64,  ReturnInt64);
	OP_DEFINE_SLOT (Bool,     boolean, SetBool,   ReturnBool);
	OP_DEFINE_SLOT (Real,     real,    SetReal,   ReturnReal);
	OP_DEFINE_SLOT (OPObject, object,  SetObject, ReturnObject);
	OP_DEFINE_SLOT (OPColorPixel, color, SetColor, ReturnColor);
	OP_DEFINE_SLOT (OPRect,   rect,    SetRect,   ReturnRect);
	OP_DEFINE_SLOT (OPText,   text,    SetText,   ReturnText);
	
	OP_DEFINE_CLASS_SPECIAL_REF (OPValue, OPObject);
};

/****************************************************************/

inline OPValueRef::OPValueRef (Card32 x)
{ this->NewValue ()->SetCard32 (x); }

inline OPValueRef::OPValueRef (Int32 x)
{ this->NewValue ()->SetInt32 (x); }

inline OPValueRef::OPValueRef (Card64 x)
{ this->NewValue ()->SetCard64 (x); }

inline OPValueRef::OPValueRef (Int64 x)
{ this->NewValue ()->SetInt64 (x); }

inline OPValueRef::OPValueRef (Bool x)
{ this->NewValue ()->SetBool (x); }

inline OPValueRef::OPValueRef (Real x)
{ this->NewValue ()->SetReal (x); }

inline OPValueRef::OPValueRef (OPColorPixelIn x)
{ this->NewValue ()->SetColor (x); }

inline OPValueRef::OPValueRef (OPRectIn x)
{ this->NewValue ()->SetRect (x); }

inline OPValueRef& OPValueRef::operator = (Card32 x)
{ this->NewValue ()->SetCard32 (x); return *this; }

inline OPValueRef& OPValueRef::operator = (Int32 x)
{ this->NewValue ()->SetInt32 (x); return *this; }

inline OPValueRef& OPValueRef::operator = (Card64 x)
{ this->NewValue ()->SetCard64 (x); return *this; }

inline OPValueRef& OPValueRef::operator = (Int64 x)
{ this->NewValue ()->SetInt64 (x); return *this; }

inline OPValueRef& OPValueRef::operator = (Bool x)
{ this->NewValue ()->SetBool (x); return *this; }

inline OPValueRef& OPValueRef::operator = (Real x)
{ this->NewValue ()->SetReal (x); return *this; }

inline OPValueRef& OPValueRef::operator = (OPColorPixelIn x)
{ this->NewValue ()->SetColor (x); return *this; }

inline OPValueRef& OPValueRef::operator = (OPRectIn x)
{ this->NewValue ()->SetRect (x); return *this; }

/****************************************************************/

inline OPValueRef::operator Card32 () const
{ return (this->object) ? ((OPValue*)(this->object))->ReturnCard32 () : 0; }

inline OPValueRef::operator Int32 () const
{ return (this->object) ? ((OPValue*)(this->object))->ReturnInt32 () : 0; }

inline OPValueRef::operator Card64 () const
{ return (this->object) ? ((OPValue*)(this->object))->ReturnCard64 () : 0; }

inline OPValueRef::operator Int64 () const
{ return (this->object) ? ((OPValue*)(this->object))->ReturnInt64 () : 0; }

inline OPValueRef::operator Bool () const
{ return (this->object) ? ((OPValue*)(this->object))->ReturnBool () : 0; }

inline OPValueRef::operator Real () const
{ return (this->object) ? ((OPValue*)(this->object))->ReturnReal () : 0; }

inline OPValueRef::operator OPColorPixel () const
{ return (this->object) ? ((OPValue*)(this->object))->ReturnColor () : OPColorPixel::none; }

inline OPValueRef::operator OPRect () const
{ return (this->object) ? ((OPValue*)(this->object))->ReturnRect () : OPRect::none; }

/****************************************************************/

extern inline Bool
operator == (OPValueIn v, Card32 x)
{ return (v.IsValid () && (v->CompareWith (x) == OP_COMP_THIS_SAME)) ? TRUE : FALSE; }

extern inline Bool
operator == (OPValueIn v, Int32 x)
{ return (v.IsValid () && (v->CompareWith (x) == OP_COMP_THIS_SAME)) ? TRUE : FALSE; }

extern inline Bool
operator == (OPValueIn v, Card64 x)
{ return (v.IsValid () && (v->CompareWith (x) == OP_COMP_THIS_SAME)) ? TRUE : FALSE; }

extern inline Bool
operator == (OPValueIn v, Int64 x)
{ return (v.IsValid () && (v->CompareWith (x) == OP_COMP_THIS_SAME)) ? TRUE : FALSE; }

extern inline Bool
operator == (OPValueIn v, Bool x)
{ return (v.IsValid () && (v->CompareWith (x) == OP_COMP_THIS_SAME)) ? TRUE : FALSE; }

extern inline Bool
operator == (OPValueIn v, Real x)
{ return (v.IsValid () && (v->CompareWith (x) == OP_COMP_THIS_SAME)) ? TRUE : FALSE; }

extern inline Bool
operator == (OPValueIn v, OPColorPixelIn x)
{ return (v.IsValid () && (v->CompareWith (x) == OP_COMP_THIS_SAME)) ? TRUE : FALSE; }

extern inline Bool
operator == (OPValueIn v, OPRectIn x)
{ return (v.IsValid () && (v->CompareWith (x) == OP_COMP_THIS_SAME)) ? TRUE : FALSE; }

/****************************************************************/

inline OPValue*
OPValueRef::CheckAndResolve () const
{
	return (OPValue*)((this->object) ? this->object->CheckAndResolve ()
									 : this->ReturnRuntime ()->ResolveNull ());
}

inline OPValue*
OPValueRef::Resolve () const
{
	return (OPValue*)((this->object) ? this->object->Resolve () : 0);
}

inline OPValue*
OPValueRef::operator -> () const
{
	return (OPValue*)(this->CheckAndResolve ());
}

/****************************************************************/

#endif /* _OPaC_VALUES_H_ */
