/*
 *	sd_grailwdo.cxx
 *
 *	GRAphical Interface Layer for building graphical interface
 *	elements. This is highly machine dependent !
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include "sd_interface.h"
#include "../common/sd_grafix.h"

#include <stdio.h>

#include <OPaC/grail.h>
#include <OPaC/new.h>
#include <OPaC/window.h>

#include <OPaC/notmacro.h>

#pragma warning (disable : 4244)				//	no warning for possible loss of data

OP_IMPLEMENTATION_FILE;

/****************************************************************/

HINSTANCE	OPaC_GrWindow::win_app;
Count		OPaC_GrWindow::counter = 0;
char        OPaC_GrWindow::globname[64] = { 0 };
HCURSOR     global_OPaC_cursor = NULL;

/****************************************************************/

#define	MAX_TITLE_LEN	200

/****************************************************************/

/*
 *	Call back function used by every Grail window. This won't
 *	do anything, apart from calling the default window proc.
 *
 *	The methods for drawing into the window are used directly
 *	in the main event loop.
 */

LRESULT CALLBACK
GrailWndProc (HWND wnd, UINT msg, WPARAM w_param, LPARAM l_param)
{
	static int pending_WM_CLOSE = 0;

	HDC      hdc;
	UINT     i;
	int      val;
	
	if (wnd == 0) {
		return DefWindowProc (wnd, msg, w_param, l_param);
	}
	
	LONG magic = GetWindowLong (wnd, 4);
	LONG inst  = GetWindowLong (wnd, 0);
	
	if ( (inst == 0)
	  || (magic != GRAIL_MAGIC) ) {
		return DefWindowProc (wnd, msg, w_param, l_param);
	}
	
	OPGrailWindow* window = 0;
	OPaC_GrWindow* winfo  = 0;
//	OPaC_ColorMan* cinfo  = 0;

	OPWindowRef win;
	
	window = (OPGrailWindow*)(inst);
	winfo  = window->ReturnPrivateWindowInfo ();
	
	switch (msg) {
		
		case WM_MOVE:
			{
				int val = DefWindowProc (wnd, msg, w_param, l_param);
				window->UserChangedGeometry ();
				return val;
			}

		case WM_CLOSE:
			if (pending_WM_CLOSE == 0) {
				
				//	Since some API functions call the window procedure directly
				//	without first generating a message for the application, we have
				//	to be sure the application gets the event (we will get it twice
				//	since it is being reposted).

				pending_WM_CLOSE = 1;
				PostMessage (wnd, msg, w_param, l_param);
				return 0;
			}

			pending_WM_CLOSE = 0;
			break;

		case WM_CREATE:
			{
				Bool visible = ((BOOL)(w_param)) ? TRUE : FALSE;
				window->SysChangedOpen (TRUE);
				window->SysChangedVisible (visible);
			}
			break;
			
//		case WM_NCCREATE:
//		case WM_NCCALCSIZE:
//		case WM_NCPAINT:
//		case WM_MOVE:

		case WM_SHOWWINDOW:
			{
				Bool visible = ((BOOL)(w_param)) ? TRUE : FALSE;
				window->SysChangedVisible (visible);
			}
			break;
		
		case WM_SETCURSOR:
			SetCursor (global_OPaC_cursor);
			break;
		
		case WM_ACTIVATE:
			{
				Bool active = (LOWORD (w_param) == WA_INACTIVE) ? FALSE : TRUE;
				Bool mini   = ((BOOL)HIWORD (w_param)) ? TRUE : FALSE;
				window->SysChangedActive (active);
				window->SysChangedReduced (mini);
			}
			break;
		
		case WM_SIZE:
			switch (w_param) {
				
				case SIZE_MINIMIZED:
					window->SysChangedReduced (TRUE);
					break;
				
				case SIZE_RESTORED:
					window->SysChangedReduced (FALSE);
					break;
			}
			
			val = DefWindowProc (wnd, msg, w_param, l_param);
			
			window->UserChangedGeometry ();
			window->GetWindow (win);
			if (win.IsValid ()) win->Display ();
			return val;

//		case WM_GETMINMAXINFO:
	
		/*
		 *	Painting can be easy... if you use double buffering
		 *	everywhere.
		 */
		
		case WM_PAINT:

			if (IsIconic (wnd)) {
					
				//	Handle iconified window... This needs special care if
				//	the window class does not specify some default ICON.
				
				window->RedrawIcon ();
				
				RECT rect;
				GetUpdateRect (wnd, & rect, TRUE);
				ValidateRect (wnd, & rect);			//	fake redraw, prevent dead loop
					
				return 0;
			}
			
			//	Window will have to redraw itself.
			
			{
				RECT rect;
				
				if (GetUpdateRect (wnd, & rect, FALSE)) {
					window->Redraw ();
				}
			}
			return 0;
		
		case WM_ERASEBKGND:
			window->Redraw ();
			return 1;
		
		/*
		 *	The palette manipulation using WINDOWS was not that
		 *	easy... It took me a full day to understand (?) the
		 *	way to do it properly. :-(
		 */
		
		case WM_PALETTEISCHANGING:
			if ((HWND) w_param == wnd)
		    	return 0;
		    
		    //	Palette is going to change. It's safer to redraw the
		    //	window.
		    
		    InvalidateRect (wnd, NULL, FALSE);
		    return 0;
		
		case WM_PALETTECHANGED:
		    if ((HWND) w_param == wnd)
		    	return 0;
			
			//	Palette of another window has changed, so we need to
			//	re-realize ours.
			
		case WM_QUERYNEWPALETTE:
		    hdc  = GetDC (wnd);
//		    hpal = SelectPalette (hdc, cinfo->hpal, FALSE);
			i    = RealizePalette (hdc);
//		    SelectPalette (hdc, hpal, FALSE);
		    ReleaseDC (wnd, hdc);
		    InvalidateRect (wnd, NULL, FALSE);
		    return 0;
		
		default:
			break;
	}
	
	return DefWindowProc (wnd, msg, w_param, l_param);
}

/****************************************************************/

/*
 *	Initialise the global variable used for attaching the
 *	windows to the application.
 */

void
OPaC_GrWindow::GlobalInit (HINSTANCE win_app)
{
	OPaC_GrWindow::win_app = win_app;
}


/****************************************************************/

OP_IMPLEMENT_CLASS (OPGrailWindow);

/****************************************************************/

OP_NOTIFY_EVENT (OPGrailWindow, Drawing);

/****************************************************************/

/*
 *	Construct a new window instance. This will build an
 *	independent window class; you might think this is
 *	expensive, but it is much easier to implement that
 *	way !
 */

OPGrailWindow::OPGrailWindow ()
{
	OP_METHOD (OPGrailWindow, 0);
	
	OPaC_GrWindow* window = OP_ALLOC_FROM_ZONE (this->ReturnMemZone (), OPaC_GrWindow);
	
	this->bounding_box.MakeEmpty ();
	this->window = window;
	this->view   = 0;
	this->style  = OP_WIN_NAKED;
	
	this->is_active  = FALSE;
	this->is_open    = FALSE;
	this->is_reduced = FALSE;
	this->is_drawing = FALSE;
	
	//	Create the 16-bit Windows window class.
	
	if (OPaC_GrWindow::globname[0] == 0) {
		snprintf (OPaC_GrWindow::globname, sizeof (OPaC_GrWindow::globname),
				  GLOBNAME_ID "%lx:%lx",
				  (long)(GetCurrentThreadId ()),
				  (long)(GetCurrentProcessId ()));
	}

	snprintf (window->wclass_name, sizeof (window->wclass_name),
			  "%s-%d", OPaC_GrWindow::globname, OPaC_GrWindow::counter++);
	
	window->wclass.style		 = CS_OWNDC; // | CS_HREDRAW | CS_VREDRAW;
	window->wclass.lpfnWndProc	 = (WNDPROC) GrailWndProc;
	window->wclass.cbClsExtra	 = 0;
	window->wclass.cbWndExtra	 = 16;
	window->wclass.hInstance	 = OPaC_GrWindow::win_app;
	window->wclass.hIcon		 = LoadIcon (NULL, IDI_APPLICATION);
	window->wclass.hCursor		 = NULL; //LoadCursor (NULL, IDC_ARROW);
	window->wclass.hbrBackground = NULL; //(HBRUSH)(COLOR_WINDOW + 1);
	window->wclass.lpszMenuName	 = NULL;
	window->wclass.lpszClassName = window->wclass_name;
	
	window->wclass_atom = RegisterClass (& window->wclass);
	window->window      = NULL;
	
	OP_ASSERT (window->wclass_atom);
}



/*
 *	Destroy a window : this will delete the window class
 *	and the window instance.
 */

OPGrailWindow::~OPGrailWindow ()
{
	OP_METHOD (~OPGrailWindow, 0);
	
	OPaC_GrWindow* window = this->window;
	
	if (window) {
		
		if (window->window) {
			
			ShowWindow (window->window, SW_HIDE);
			DestroyWindow (window->window);
		}
		
		UnregisterClass (window->wclass_name, OPaC_GrWindow::win_app);
		
		OP_FREE_FROM_ZONE (this->ReturnMemZone (), window);
	}
}



void
OPGrailWindow::StoreInstance (OPStorageIn storage)
{
	OP_METHOD (StoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	OP_ABORT ("Trying to save Grail Window");
}



void
OPGrailWindow::RestoreInstance (OPStorageIn storage)
{
	OP_METHOD (RestoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	OP_ABORT ("Trying to restore Grail Window");
}



/*
 *	Create a window with a title bar and a minimum adorn-
 *	ment, depending on the specified style.
 *
 *	NB: The top left corner of the window will be [x1, y1]
 *	    and the client size (the part you can draw into)
 *		is specified by x2-x1+1 and y2-y1+1.
 */

void
OPGrailWindow::Initialise (OPWindowIn view,
						   OPWindowStyle style,
						   OPStringIn name,
						   OPRectIn box, OPRelCoord margin)
{
	OP_METHOD (Initialise, 5);
	OP_ARG (1, OPWindowIn, view);
	OP_ARG (2, OPWindowStyle, style);
	OP_ARG (3, OPStringIn, name);
	OP_ARG (4, OPRectIn, box);
	OP_ARG (5, OPRelCoord, margin);
	OP_ASSERT ((OPRelCoord)(box.DX ()) > (2*margin));
	OP_ASSERT ((OPRelCoord)(box.DY ()) > (2*margin));
	
	OPCoord x1 = box.X1 () + margin;
	OPCoord y1 = box.Y1 () + margin;
	OPCoord x2 = box.X2 () - margin;
	OPCoord y2 = box.Y2 () - margin;
	
	this->Initialise (view, style, name, x1, y1, x2, y2);
}

void
OPGrailWindow::Initialise (OPWindowIn view,
						   OPWindowStyle style, OPStringIn name,
						   OPCoord x1, OPCoord y1, OPCoord x2, OPCoord y2)
{
	OP_METHOD (Initialise, 7);
	OP_ARG (1, OPWindowIn, view);
	OP_ARG (2, OPWindowStyle, style);
	OP_ARG (3, OPStringIn, name);
	OP_ARG (4, OPCoord, x1);
	OP_ARG (5, OPCoord, y1);
	OP_ARG (6, OPCoord, x2);
	OP_ARG (7, OPCoord, y2);
	OP_ASSERT ((x2 > x1) && (y2 > y1));
	
	HWND  window   = NULL;
	DWORD ws       = 0;
	DWORD ex_style = 0;
	
	//	Translate the window mode :
	
	switch (style & OP_WIN_MODE_MASK) {
		
		case OP_WIN_NAKED:
			ws       = WS_POPUP;
			ex_style = WS_EX_TOOLWINDOW;
			break;
		
		case OP_WIN_NORMAL:
			ws = WS_POPUP | WS_BORDER;
			break;
		
		case OP_WIN_TITLE:
			ws = WS_POPUP | WS_BORDER | WS_CAPTION;
			break;
	}
	
	
	//	Translate the window attributes :
	
	if (style & OP_WIN_MOVE_OK)		ws |= WS_BORDER | WS_CAPTION;
	if (style & OP_WIN_SIZE_OK)		ws |= WS_THICKFRAME | WS_CAPTION | WS_MAXIMIZEBOX;
	if (style & OP_WIN_REDUCE_OK)	ws |= WS_CAPTION | WS_MINIMIZEBOX;
	if (style & OP_WIN_CLOSE_OK)	ws |= WS_CAPTION | WS_SYSMENU;
	
	if (style & OP_WIN_TRANSPARENT)	ex_style |= WS_EX_TRANSPARENT;
	if (style & OP_WIN_FLOAT_TOOL)	ex_style |= WS_EX_TOOLWINDOW;
	if (style & OP_WIN_TOP_LEVEL)	ex_style |= WS_EX_TOPMOST;
	
	this->style = style;
	
	//	Now create the corresponding window. We have to extract the window name
	//	and convert it to ISO Latin-1 encoding so that Win16 will be happy with
	//	it.
	
	OPCoord xl = 0;
	OPCoord xr = 0;
	OPCoord yt = 0;
	OPCoord yb = 0;
	
	OPGrailWindow::GetFrameSize (style, xl, yt, xr, yb);
	
	LatinChar name_copy[MAX_TITLE_LEN];
	Count     length = name.IsValid () ? name->CountCharacters () : 0;
	
	name->GetStringCopy (name_copy, MAX_TITLE_LEN);
	
	window = CreateWindowEx ( WS_EX_ACCEPTFILES | ex_style,
							  this->window->wclass_name,
							  name_copy, ws,
							  (int)(x1-xl),
							  (int)(y1-yt),
							  (int)(x2-x1 + xl+xr + 1),
							  (int)(y2-y1 + yt+yb + 1),
							  NULL, NULL,
							  OPaC_GrWindow::win_app,
							  NULL );
	
	OP_ASSERT (window);
	
	SetWindowLong (window,  0, (LONG)(OPGrailWindow*)(this));
	SetWindowLong (window,  4, GRAIL_MAGIC);
	SetWindowLong (window,  8, 0);
	SetWindowLong (window, 12, 0);
	
	this->window->window = window;
	this->view = view;
	this->bounding_box = OPRect (x1, y1, x2, y2);
	
	HDC hdc = GetDC (this->window->window);
//	HPALETTE hpal_new = this->color_man->ReturnPrivateInfo ()->hpal;
	
//	if (hpal_new) {
//		SelectPalette (hdc, hpal_new, FALSE);
//	}
	
	RealizePalette (hdc);
	ReleaseDC (this->window->window, hdc);
	
	super::Initialise (this, x2-x1 + 1, y2-y1 + 1);
}
	


/*
 *	Select another name for the current window. This is only
 *	possible if the window has been defined with a title bar.
 */

void
OPGrailWindow::ChangeName (OPStringIn name)
{
	OP_METHOD (ChangeName, 1);
	OP_ARG (1, OPStringIn, name);
	
	//	Convert the name to ISO Latin-1 encoding to keep Windows happy.
	
	LatinChar name_copy[200];
	Count     length = name.IsValid () ? name->CountCharacters ()+1 : 0;
	
	if (length) {
		if (length > sizeof (name_copy)) {
			length = sizeof (name_copy);
		}
		name->GetStringCopy (name_copy, length);
	} else {
		name_copy[0] = 0;
	}
	
	SetWindowText (this->window->window, name_copy);	
}



void
OPGrailWindow::Open ()
{
	OP_METHOD (Open, 0);
	
	if ( (this->is_open == TRUE)
	  && (this->is_reduced == FALSE) )
		return;
	
	ShowWindow (this->window->window, this->is_active ? SW_SHOW : SW_SHOWNOACTIVATE);

	//	When opening a transparent window, we must copy the contents of the window
	//	to the bitmap, otherwise the next update would ouverwrite the "transparency"
	//	with the background of the bitmap.
	
	if (this->style & OP_WIN_TRANSPARENT) {
		
		//	NIY
	}
	
	UpdateWindow (this->window->window);
	
	this->is_open    = TRUE;
	this->is_reduced = FALSE;
}



void
OPGrailWindow::Reduce ()
{
	OP_METHOD (Reduce, 0);
	
	if ( (this->is_open == TRUE)
	  && (this->is_reduced == TRUE) )
		return;
	
	ShowWindow (this->window->window, SW_MINIMIZE);
	UpdateWindow (this->window->window);
	
	this->is_open    = TRUE;
	this->is_reduced = TRUE;
}



void
OPGrailWindow::Close ()
{
	OP_METHOD (Close, 0);
	
	if (this->is_open == FALSE)
		return;
	
	ShowWindow (this->window->window, SW_HIDE);
	
	this->is_open    = FALSE;
	this->is_reduced = FALSE;
}


void
OPGrailWindow::ActivateAsFrontWindow (Bool front)
{
	if (this->is_open) {
		if (this->is_reduced) {
			return;
		}
		
		if (front) {
			if (this->style & OP_WIN_TOP_LEVEL)	{
				SetWindowPos (this->window->window, HWND_TOPMOST, 0, 0, 0, 0,
							  SWP_NOSIZE | SWP_NOMOVE);
			} else {
				SetWindowPos (this->window->window, HWND_NOTOPMOST, 0, 0, 0, 0,
							  SWP_NOSIZE | SWP_NOMOVE);
				SetWindowPos (this->window->window, HWND_TOP, 0, 0, 0, 0,
							  SWP_NOSIZE | SWP_NOMOVE);
			}
		} else {
			SetWindowPos (this->window->window, HWND_TOP, 0, 0, 0, 0,
						  SWP_NOSIZE | SWP_NOMOVE | SWP_NOZORDER);
		}
	}
}


void
OPGrailWindow::SysChangedActive (Bool yes)
{
	OP_METHOD (SysChangedActive, 1);
	OP_ARG (1, Bool, yes);
	
	this->is_active = yes;
	view->SysChangedActive (yes);
}

void
OPGrailWindow::SysChangedOpen (Bool yes)
{
	OP_METHOD (SysChangedOpen, 1);
	OP_ARG (1, Bool, yes);
	
	this->is_open = yes;
	view->SysChangedOpen (yes);
}

void
OPGrailWindow::SysChangedVisible (Bool yes)
{
	OP_METHOD (SysChangedVisible, 1);
	OP_ARG (1, Bool, yes);
	
	view->SysChangedVisible (yes);
}
 
void
OPGrailWindow::SysChangedReduced (Bool yes)
{
	OP_METHOD (SysChangedReduced, 1);
	OP_ARG (1, Bool, yes);
	
	this->is_reduced = yes;
	view->SysChangedReduced (yes);
}

	
Bool
OPGrailWindow::IsActive () const
{
	OP_METHOD (IsActive, 0);
	
	return this->is_active;
}


/*
 *	Set window's origin. This will be reflected immediately
 *	by the window manager so that this can be used for
 *	dragging.
 */

void
OPGrailWindow::SetOrigin (OPCoord x, OPCoord y)
{
	OP_METHOD (SetOrigin, 2);
	OP_ARG (1, OPCoord, x);
	OP_ARG (2, OPCoord, y);
	
	if ( (this->window)
	  && (this->window->window) ) {
		
		this->bounding_box.SetOrigin (x, y);
		
		OPCoord xl = 0;
		OPCoord xr = 0;
		OPCoord yt = 0;
		OPCoord yb = 0;
		
		OPGrailWindow::GetFrameSize (style, xl, yt, xr, yb);
		
		MoveWindow (this->window->window,
					(int)(this->bounding_box.X1 ()-xl),
					(int)(this->bounding_box.Y1 ()-yt),
					(int)(this->bounding_box.DX ()+xl+xr),
					(int)(this->bounding_box.DY ()+yt+yb), 1);
	}
}



/*
 *	Return the window's origin.
 */

void
OPGrailWindow::GetOrigin (OPCoordOut x, OPCoordOut y) const
{
	OP_METHOD (GetOrigin, 2);
	OP_ARG (1, OPCoordOut, x);
	OP_ARG (2, OPCoordOut, y);
	
	x = this->bounding_box.X1 ();
	y = this->bounding_box.Y1 ();
}



/*
 *	Set the window's size.
 */

void
OPGrailWindow::SetSize (OPCoord dx, OPCoord dy)
{
	OP_METHOD (SetSize, 2);
	OP_ARG (1, OPCoord, dx);
	OP_ARG (2, OPCoord, dy);

	if ( (this->bounding_box.DX () == dx)
	  && (this->bounding_box.DY () == dy) ) {
		return;
	}
	
	if ( (this->window)
	  && (this->window->window) ) {
		
		this->bounding_box.SetSize (dx, dy);
		
		OPCoord xl = 0;
		OPCoord xr = 0;
		OPCoord yt = 0;
		OPCoord yb = 0;
		
		OPGrailWindow::GetFrameSize (style, xl, yt, xr, yb);

#if 0
		MoveWindow (this->window->window,
					(int)(this->bounding_box.X1 ()-xl),
					(int)(this->bounding_box.Y1 ()-yt),
					(int)(this->bounding_box.DX ()+xl+xr),
					(int)(this->bounding_box.DY ()+yt+yb), 1);
#else
		SetWindowPos (this->window->window, NULL,
					  (int)(this->bounding_box.X1 ()-xl),
					  (int)(this->bounding_box.Y1 ()-yt),
					  (int)(this->bounding_box.DX ()+xl+xr),
					  (int)(this->bounding_box.DY ()+yt+yb),
					  SWP_NOACTIVATE | SWP_NOOWNERZORDER);
		UpdateWindow (this->window->window);
#endif
		
		this->dx = this->bounding_box.DX ();
		this->dy = this->bounding_box.DY ();
		
		this->surface->SetNewSize (this->dx, this->dy);
	}
}



/*
 *	Return the window's size.
 */

void
OPGrailWindow::GetSize (OPCoordOut dx, OPCoordOut dy) const
{
	OP_METHOD (GetOrigin, 2);
	OP_ARG (1, OPCoordOut, dx);
	OP_ARG (2, OPCoordOut, dy);
	
	dx = this->bounding_box.DX ();
	dy = this->bounding_box.DY ();
}


/*
 *	The user either resized or moved the window, or both,
 *	so we have to take care of it.
 */

void
OPGrailWindow::UserChangedGeometry ()
{
	OP_METHOD (UserChangedGeometry, 0);
	
	WINDOWPLACEMENT placement;
	RECT            rect;
	placement.length = sizeof (WINDOWPLACEMENT);
	if ( (GetWindowPlacement (this->window->window, & placement))
	  && (placement.showCmd == SW_SHOWMINIMIZED) ) {
		return;
	}

	OPRect box = this->bounding_box;
	
	GetWindowRect (this->window->window, & rect);

	OPCoord xl = 0;
	OPCoord xr = 0;
	OPCoord yt = 0;
	OPCoord yb = 0;
	
	OPGrailWindow::GetFrameSize (style, xl, yt, xr, yb);
	
	this->bounding_box.X1 () = (OPCoord)(rect.left) + xl;
	this->bounding_box.Y1 () = (OPCoord)(rect.top)  + yt;
	this->bounding_box.X2 () = (OPCoord)(rect.right) - xr - 1;
	this->bounding_box.Y2 () = (OPCoord)(rect.bottom) - yb - 1;
	
	if (box != this->bounding_box) {
		
		this->dx = this->bounding_box.DX ();
		this->dy = this->bounding_box.DY ();
		
		this->surface->SetNewSize (this->dx, this->dy);
		
		this->view->UserChangedGeometry (this->bounding_box);
	}
}

/*
 *	Return a window's frame size. This is style dependent
 *	and may be different for the left, right, top and
 *	bottom borders.
 */

void
OPGrailWindow::GetFrameSize (OPCoordOut xl, OPCoordOut yt,
							 OPCoordOut xr, OPCoordOut yb)
{
	OP_METHOD (GetFrameSize, 4);
	OP_ARG (1, OPCoordOut, xl);
	OP_ARG (2, OPCoordOut, yt);
	OP_ARG (3, OPCoordOut, xr);
	OP_ARG (4, OPCoordOut, yb);
	
	OPGrailWindow::GetFrameSize (this->style, xl, yt, xr, yb);
}



/*
 *	Return a window's frame size. This is style dependent
 *	and may be different for the left, right, top and
 *	bottom borders. This is a static member !
 */

void
OPGrailWindow::GetFrameSize (OPWindowStyle style,
							 OPCoordOut xl, OPCoordOut yt,
							 OPCoordOut xr, OPCoordOut yb)
{
	OP_MEMBER (GetFrameSize, 5);
	OP_ARG (1, OPWindowStyle, style);
	OP_ARG (2, OPCoordOut, xl);
	OP_ARG (3, OPCoordOut, yt);
	OP_ARG (4, OPCoordOut, xr);
	OP_ARG (5, OPCoordOut, yb);
	
	xl = xr = 0;
	yt = yb = 0;
	
	if (style & (OP_WIN_TITLE | OP_WIN_MOVE_OK | OP_WIN_REDUCE_OK | OP_WIN_CLOSE_OK)) {
		if (style & OP_WIN_FLOAT_TOOL) {
			yt = (OPCoord)(GetSystemMetrics (SM_CYSMCAPTION));
		} else {
			yt = (OPCoord)(GetSystemMetrics (SM_CYCAPTION));
		}
	}
	
	if (style & OP_WIN_SIZE_OK) {
		yt += (OPCoord)(GetSystemMetrics (SM_CYFRAME));
	} else if (style & OP_WIN_TITLE) {
		yt += (OPCoord)(GetSystemMetrics (SM_CYEDGE));
		yt += (OPCoord)(GetSystemMetrics (SM_CYBORDER));
	} else if (style & OP_WIN_NORMAL) {
		yt += (OPCoord)(GetSystemMetrics (SM_CYBORDER));
	}
	
	if (style & OP_WIN_SIZE_OK) {
		
		xl = xr = (OPCoord)(GetSystemMetrics (SM_CXFRAME));
		yb = (OPCoord)(GetSystemMetrics (SM_CYFRAME));
		
	} else if (style & (OP_WIN_TITLE | OP_WIN_MOVE_OK | OP_WIN_REDUCE_OK | OP_WIN_CLOSE_OK)) {
		
		xl = xr = (OPCoord)(GetSystemMetrics (SM_CXEDGE)) + (OPCoord)(GetSystemMetrics (SM_CXBORDER));
		yb = (OPCoord)(GetSystemMetrics (SM_CYEDGE)) + (OPCoord)(GetSystemMetrics (SM_CYBORDER));
		
	} else if (style & OP_WIN_NORMAL) {
		xl = xr = (OPCoord)(GetSystemMetrics (SM_CXBORDER));
		yb = (OPCoord)(GetSystemMetrics (SM_CYBORDER));
	}
}



/*
 *	Start drawing into the window. This will first draw into
 *	the buffering pixmap.
 */

void
OPGrailWindow::BeginDraw (OPRectIn box, OPRelCoord margin)
{
	OP_METHOD (BeginDraw, 2);
	OP_ARG (1, OPRectIn, box);
	OP_ARG (2, OPRelCoord, margin);
	super::BeginDraw (box, margin);
}



/*
 *	Start drawing into the window. This will first draw into
 *	the buffering pixmap.
 */

void
OPGrailWindow::BeginDraw (OPCoord x1, OPCoord y1, OPCoord x2, OPCoord y2)
{
	OP_METHOD (BeginDraw, 4);
	OP_ARG (1, OPCoord, x1);
	OP_ARG (2, OPCoord, y1);
	OP_ARG (3, OPCoord, x2);
	OP_ARG (4, OPCoord, y2);
	super::BeginDraw (x1, y1, x2, y2);
}



/*
 *	End of drawing. Invalidate the drawing rectangle in the
 *	window on screen.
 */

void
OPGrailWindow::EndDraw ()
{
	OP_METHOD (EndDraw, 0);
	super::EndDraw ();
	
	const RECT rect =
	{
		(int)(this->rect_x1),
		(int)(this->rect_y1),
		(int)(this->rect_x2 + 1),
		(int)(this->rect_y2 + 1)
	};
	
	InvalidateRect (this->window->window, & rect, FALSE);
}



/*
 *	Send the graphic output to the screen. This will update
 *	the part of the window which has been marked as invalid
 *	using the BeginDraw/EndDraw methods.
 */

void
OPGrailWindow::Flush ()
{
	OP_METHOD (Flush, 0);
	super::Flush ();
	UpdateWindow (this->window->window);
}


/*
 *	Redraw the window.
 */

void
OPGrailWindow::Redraw ()
{
	OP_METHOD (Redraw, 0);
	
	PAINTSTRUCT paint;
	HDC hdc_window = BeginPaint (this->window->window, &paint);
	
	this->is_drawing = TRUE;
	
	int x  = paint.rcPaint.left;
	int y  = paint.rcPaint.top;
	int dx = paint.rcPaint.right - paint.rcPaint.left + 1;
	int dy = paint.rcPaint.bottom - paint.rcPaint.top + 1;
	
	this->draw_rect.SetOrigin (x, y);
	this->draw_rect.SetSize (dx, dy);
	
//	HPALETTE hpal_new = this->color_man->ReturnPrivateInfo ()->hpal;
	
//	if (hpal_new) {
//		SelectPalette (hdc_window, hpal_new, FALSE);
//	}
	
//	if (RealizePalette (hdc_window)) {
//		InvalidateRect (this->window->window, NULL, FALSE);
//	}
	
	BITMAPINFO info;
	
	info.bmiHeader.biSize          = sizeof (info.bmiHeader);
	info.bmiHeader.biWidth         = (LONG)(this->surface->Pitch () / 4);
	info.bmiHeader.biHeight        = -(LONG)(this->dy);
	info.bmiHeader.biPlanes        = 1;
	info.bmiHeader.biBitCount      = 32;
	info.bmiHeader.biCompression   = BI_RGB;
	info.bmiHeader.biSizeImage     = 0;
	info.bmiHeader.biXPelsPerMeter = 0;
	info.bmiHeader.biYPelsPerMeter = 0;
	info.bmiHeader.biClrUsed       = 0;
	info.bmiHeader.biClrImportant  = 0;
	
	//	Pentium 133MHz, 32 bpp, 1000 x 1000 => 11ms
	//					16 bpp, 1000 x 1000 =>  7ms
	
	void* bits = this->surface->Line (0);
	
	if (this->partition_count) {
		
		//	If the space has been partitioned, make sure we draw only the
		//	interesting parts of the window (those which contain the real
		//	views).
		
		for (Count index = 0; index < this->partition_count; index++) {
			
			OPRect box = this->draw_rect;
			box *= this->partition_array[index];
			
			if (! box.IsEmpty ()) {
				
				x  = (int)(box.X1 ());
				y  = (int)(box.Y1 ());
				dx = (int)(box.DX ());
				dy = (int)(box.DY ());
				
				SetDIBitsToDevice (hdc_window, x, y, dx, dy, x, this->dy-dy-y,
								   0, this->dy, bits, &info, DIB_RGB_COLORS);
			}
		}
		
	} else {
		
		SetDIBitsToDevice (hdc_window, x, y, dx, dy, x, this->dy-dy-y,
						   0, this->dy, bits, &info, DIB_RGB_COLORS);
	}
	
	this->Report (notify_Drawing);
	this->is_drawing = FALSE;
	
	EndPaint (this->window->window, &paint);
}


/*
 *	Fetch the current drawing rectangle (if drawing is in
 *	progress), otherwise return FALSE.
 */

Bool
OPGrailWindow::GetDrawRect (OPRectOut box) const
{
	OP_METHOD (GetDrawRect, 1);
	OP_ARG (1, OPRectOut, box);
	
	if (this->is_drawing) {
		box = this->draw_rect;
		return TRUE;
	}
	
	return FALSE;
}


/*
 *	Redraw the icon (miniaturized window).
 */

void
OPGrailWindow::RedrawIcon ()
{
	OP_METHOD (RedrawIcon, 0);
	
	//	NIY
}



struct GrailWdoEnumData
{
	HWND		hwnd;
	HWND		skip;
	Count		num;
	OPCoord		x;
	OPCoord		y;
	OPaC_RemoteAppDesc* desc;
	Bool		ok;
	const char*	name;
	OPGrailWindow* wdo;
};

static Bool
CheckOPaCWindow (GrailWdoEnumData* data)
{
	RECT rect;
	char name[128];
	LONG inst    = GetWindowLong (data->hwnd, 0);
	LONG magic   = GetWindowLong (data->hwnd, 4);
	Bool ok_rect = GetWindowRect (data->hwnd, &rect);
	Bool ok_name = GetClassName (data->hwnd, name, 127) != 0;
	
	if ( (inst != 0)
	  && (magic == GRAIL_MAGIC)
	  && (ok_rect) && (ok_name)
  	  && (rect.left < data->x) && (rect.right > data->x)
	  && (rect.top < data->y)  && (rect.bottom > data->y)
	  && (strncmp (data->name, name, strlen (data->name)) == 0) ) {
		
		if (data->desc) {
			data->desc->hwnd = data->hwnd;
			nstrcpy (data->desc->name, name, sizeof (data->desc->name));
		}
		
		data->wdo = (OPGrailWindow*)(inst);
		
		return TRUE;
	}
	
	return FALSE;
}

static BOOL CALLBACK
GrailWdoEnumChildProc (HWND hwnd, LPARAM param)
{
	GrailWdoEnumData* data = (GrailWdoEnumData*)(param);
	
	data->hwnd = hwnd;
	
	if (data->skip != hwnd) {
		if (CheckOPaCWindow (data)) {
			if (data->num-- == 0) {
				data->ok = TRUE;
				return FALSE;
			}
		}
	}
	
	return TRUE;
}



/*
 *	Find which OPaC window -- if any -- corresponds to the specified
 *	coordinates. Look for the nth window (0 is the topmost).
 */

Bool
OPGrailWindow::FindOwnWindow (OPCoord x, OPCoord y, OPGrailWindowOut wdo,
							  Count index)
{
	OP_MEMBER (FindOwnWindow, 4);
	OP_ARG (1, OPCoord, x);
	OP_ARG (2, OPCoord, y);
	OP_ARG (3, OPGrailWindowOut, wdo);
	OP_ARG (4, Count, index);

	POINT pt    = { (LONG)(x), (LONG)(y) };
	HWND  found = WindowFromPoint (pt);

	GrailWdoEnumData data = { found, found, index, x, y, 0, FALSE, OPaC_GrWindow::globname, 0 };
	
	if (found) {
		if (CheckOPaCWindow (&data)) {	
			if (data.num-- == 0) {
				wdo = data.wdo;
				OP_TRACE ("wdo found: %08x\n", (long)(wdo.ReturnRawAddress ()));
				return TRUE;
			}
		}
	}
	
	EnumChildWindows (NULL, GrailWdoEnumChildProc, (LPARAM)(&data));
	
	if (data.ok) {
		wdo = data.wdo;
		OP_TRACE ("wdo found: %08x\n", (long)(wdo.ReturnRawAddress ()));
	}
	
	return data.ok;
}

Bool
OPGrailWindow::FindOPaCWindow (OPCoord x, OPCoord y, OPaC_RemoteAppDesc* desc,
							   Count index)
{
	OP_MEMBER (FindOPaCWindow, 4);
	OP_ARG (1, OPCoord, x);
	OP_ARG (2, OPCoord, y);
	OP_ARG (3, OPaC_RemoteAppDesc*, desc);
	OP_ARG (4, Count, index);
	
	if (desc == 0) {
		return FALSE;
	}
	
	POINT pt    = { (LONG)(x), (LONG)(y) };
	HWND  found = WindowFromPoint (pt);

	GrailWdoEnumData data = { found, found, index, x, y, desc, FALSE, GLOBNAME_ID, 0 };
	
	if (found) {
		if (CheckOPaCWindow (&data)) {	
			if (data.num-- == 0) return TRUE;
		}
	}
	
	EnumChildWindows (NULL, GrailWdoEnumChildProc, (LPARAM)(&data));
	
	return data.ok;
}


/*
 *	Get the position of the specified display. If the display does
 *	not exist, return FALSE.
 */

Bool
OPGrailWindow::GetDisplayBox (OPRectOut box, Count display)
{
	OP_MEMBER (GetDisplayBox, 2);
	OP_ARG (1, OPRectOut, box);
	OP_ARG (2, Count, display);
	
	//	NIY: multiple-monitor support (NT 5 and Windows 98)
	
	if (display > 0) return FALSE;
	
	RECT rect;
	
	HWND desktop = GetDesktopWindow ();
	GetWindowRect (desktop, &rect);
	
	box.SetOrigin (0, 0);
	box.SetSize (rect.right, rect.bottom);
	
	return TRUE;
}

Bool
OPGrailWindow::GetDisplayFromPoint (OPCoord x, OPCoord y, CountOut display)
{
	OP_MEMBER (GetDisplayFromPoint, 3);
	OP_ARG (1, OPCoord, x);
	OP_ARG (2, OPCoord, y);
	OP_ARG (3, CountOut, display);
	
	//	NIY: multiple-monitor support (NT 5 and Windows 98)
	
	RECT rect;
	
	HWND desktop = GetDesktopWindow ();
	GetWindowRect (desktop, &rect);
	
	int px = (int)(x);
	int py = (int)(y);
	
	if ( (px >= 0) && (px <= rect.right)
	  && (py >= 0) && (py <= rect.bottom) ) {
		display = 0;
		return TRUE;
	}
	
	return FALSE;
}
