/*
 *	sd_thread.cxx
 *
 *	Architecture dependent thread manipulation class.
 *
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include "sd_interface.h"

#include <OPaC/performer.h>
#include <OPaC/thread.h>

/****************************************************************/

OP_IMPLEMENTATION_FILE;

/****************************************************************/

static DWORD WINAPI
ThreadProc (LPVOID param)
{
	OPThread::ThreadLauncher (param);
	return 0;
}

/****************************************************************/

Bool
OPThread::Run ()
{
	OP_METHOD (Run, 0);
	
	if (this->IsRunning ()) {
		return FALSE;
	}
	
	this->is_running = FALSE;
	this->is_dead    = FALSE;
	
	DWORD  thread_id = 0;
	HANDLE thread_h  = CreateThread (NULL, 0, ThreadProc, this, 0, &thread_id);
	
	if (thread_h == NULL) {
		return FALSE;
	}
	
	return TRUE;
}


void
OPThread::ThreadLauncher (void* arg)
{
	OPThreadRef thread = (OPThread*)(arg);
	
	thread->is_running = TRUE;
	thread->is_dead    = FALSE;
	
	thread->performer->Perform (thread);
	
	thread->is_running = FALSE;
	thread->is_dead    = TRUE;
}

/****************************************************************/
