/*
 *	debughook.cxx
 *
 *	This file defines the Debug Hook class, which allows to set
 *	debugging hooks to access internal events.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include <OPaC/debughook.h>

/****************************************************************/

OP_IMPLEMENTATION_FILE;
OP_IMPLEMENT_CLASS (OPDebugHook);

/****************************************************************/

/*
 *	
 */

OPDebugHook::OPDebugHook ()
{
	OP_METHOD (OPDebugHook, 0);
	
}



/*
 *	
 */

OPDebugHook::~OPDebugHook ()
{
	OP_METHOD (~OPControl, 0);
	
}



/*
 *	Mark the object for deletion
 */

void
OPDebugHook::Delete ()
{
	OP_METHOD (Delete, 0);
	
	super::Delete ();
}



/*
 *	Save the debug hook to the storage.
 */

void
OPDebugHook::StoreInstance (OPStorageIn storage)
{
	OP_METHOD (StoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	
	super::StoreInstance (storage);
	
	static OPStorageSync info ("<OPDebugHook>");
	
	storage << info;
	storage << OPStorageSync::begin;
	storage << OPStorageSync::end;
}



/*
 *	Restore the debug hook from the storage.
 */

void
OPDebugHook::RestoreInstance (OPStorageIn storage)
{
	OP_METHOD (RestoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	
	super::RestoreInstance (storage);
	
	storage >> OPStorageSync::type;
	storage >> OPStorageSync::begin;
	storage >> OPStorageSync::end;
}


