/*
 *	dict.cxx
 *
 *	Sorted dictionary class. This is NOT optimized at all and
 *	will have to be implemented in a better way asap.
 *	
 *	I have got no time for it now :^( !!!
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include <OPaC/container.h>
#include <OPaC/dictit.h>
#include <OPaC/new.h>

/****************************************************************/

OP_IMPLEMENTATION_FILE;
OP_IMPLEMENT_CLASS (OPDict);

/****************************************************************/


/*
 *	Create an empty dictionary.
 */

OPDict::OPDict ()
{
	OP_METHOD (OPDict, 0);
	
	this->head = 0;
	this->tail = 0;
}



/*
 *	Destroy the dictionary.
 */

OPDict::~OPDict ()
{
	OP_METHOD (~OPDict, 0);
	
	this->RemoveAll ();
}



/*
 *	Save the dictionary to the storage.
 */

void
OPDict::StoreInstance (OPStorageIn storage)
{
	OP_METHOD (StoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	
	super::StoreInstance (storage);
	
	static OPStorageSync info ("<OPDict { <elem key value> } >");
	
	storage << info;
	storage << OPStorageSync::begin;
	
	Element* elem = this->head;
	
	storage << OPStorageSync::repeat;
	
	while (elem) {
		storage << OPStorageSync::begin;
		storage << elem->key;
		storage << elem->value;
		storage << OPStorageSync::end;
		elem = elem->next;
	}
	
	storage << OPStorageSync::end_repeat;
	storage << OPStorageSync::end;
}



/*
 *	Restore the dictionary from the storage.
 */

void
OPDict::RestoreInstance (OPStorageIn storage)
{
	OP_METHOD (RestoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	OP_ASSERT (this->head == 0);
	OP_ASSERT (this->tail == 0);
	
	super::RestoreInstance (storage);
	
	storage >> OPStorageSync::type;
	storage >> OPStorageSync::begin;
	
	Element* elem = 0;
	Count    num  = this->num_element;
	
	OPMemZone* zone = this->ReturnMemZone ();
	
	storage >> OPStorageSync::repeat;
	
	while (num--) {
		
		elem = NEW_FROM_ZONE (zone) Element;
		
		elem->next  = 0;
		elem->prev  = this->tail;
		elem->key   = 0;
		elem->value = 0;
		
		(elem->next ? elem->next->prev : this->tail) = elem;
		(elem->prev ? elem->prev->next : this->head) = elem;
		
		storage >> OPStorageSync::begin;
		storage >> elem->key;
		storage >> elem->value;
		storage >> OPStorageSync::end;
	}
	
	storage >> OPStorageSync::end_repeat;
	storage >> OPStorageSync::end;
}



/*
 *	Prepare the dictionary for its destruction.
 */

void
OPDict::Delete ()
{
	OP_METHOD (Delete, 0);
	
	this->RemoveAll ();
	super::Delete ();
}

OPIteratorRef
OPDict::NewIterator ()
{
	OPDictIteratorRef iterator = OP_NEW (OPDictIterator);
	
	this->InsertIterator (iterator);
	this->iterators->Insert (iterator);
	return iterator;
}



/*
 *	Insert a key/value pair into the dictionary.
 */

Bool
OPDict::Insert (OPObjectIn key, OPObjectIn value)
{
	OP_METHOD (Insert, 2);
	OP_ARG (1, OPObjectIn, key);
	OP_ARG (2, OPObjectIn, value);
	
	Element* find = this->head;
	
	//	Find the element which will become the successor in the dictionary...
	//	This is quite ineffective - should try to use splay or Red-Black trees.
	
	if (key.IsValid ()) {
		while (find && find->key.IsValid ()) {
			if (find->key->CompareWith (key) == OP_COMP_THIS_AFTER) {
				break;
			}
			find = find->next;
		}
	}
	
	Element* elem = NEW_FROM_ZONE (this->ReturnMemZone ()) Element;
	
	elem->next  = (find) ? find : 0;
	elem->prev  = (find) ? find->prev : this->tail;
	elem->key   = key;
	elem->value = value;
	
	(elem->prev ? elem->prev->next : this->head) = elem;
	(elem->next ? elem->next->prev : this->tail) = elem;
	
	this->num_element++;
	
	if (this->iterators.IsValid ()) {
		Count index = 0;
		OPDictIteratorRef iter;
		
		while (this->iterators->Get (index++, iter)) {
			if (iter.IsValid ()) {
				iter->NotifyInsert (elem);
			}
		}
	}
	
	return TRUE;
}



/*
 *	Remove an element from the dictionary.
 */

Bool
OPDict::Remove (Count pos)
{
	OP_METHOD (Remove, 1);
	OP_ARG (1, Count, pos);
	
	if (pos >= this->num_element) {
		return FALSE;
	}
	
	Element* elem = this->head;
	
	while (pos--) {
		elem = elem->next;
	}
	
	if (this->iterators.IsValid ()) {
		Count index = 0;
		OPDictIteratorRef iter;
		
		while (this->iterators->Get (index++, iter)) {
			if (iter.IsValid ()) {
				iter->NotifyRemove (elem);
			}
		}
	}
	
	(elem->next ? elem->next->prev : this->tail) = elem->prev;
	(elem->prev ? elem->prev->next : this->head) = elem->next;
	
	this->num_element--;
	
	elem->key   = 0;
	elem->value = 0;
	
	DELETE (elem);
	
	return TRUE;
}



/*
 *	Remove an object specified by its key.
 */

Bool
OPDict::Remove (OPObjectIn key, Count nth)
{
	OP_METHOD (Remove, 2);
	OP_ARG (1, OPObjectIn, key);
	OP_ARG (2, Count, nth);
	
	Element* elem = this->head;
	
	if (key.IsValid () == FALSE) {
		return FALSE;
	}
	
	while (elem) {
		
		if ( (elem->key == key)
		  || ( (elem->key.IsValid ())
		    && (elem->key->CompareWith (key) == OP_COMP_THIS_SAME) ) ) {
			
			if (nth == 0) {
				
				if (this->iterators.IsValid ()) {
					Count index = 0;
					OPDictIteratorRef iter;
					
					while (this->iterators->Get (index++, iter)) {
						if (iter.IsValid ()) {
							iter->NotifyRemove (elem);
						}
					}
				}
				
				(elem->next ? elem->next->prev : this->tail) = elem->prev;
				(elem->prev ? elem->prev->next : this->head) = elem->next;
				
				this->num_element--;
				
				elem->key   = 0;
				elem->value = 0;
				
				DELETE (elem);
			    
			    return TRUE;
			}
			
			nth--;
		}
		
		elem = elem->next;
	}
	
	return FALSE;
}



/*
 *	Remove every object from the dictionary.
 */

void
OPDict::RemoveAll ()
{
	OP_METHOD (RemoveAll, 0);
	
	Element* elem = this->head;
	Element* next = 0;
	
	while (elem) {
		
		if (this->iterators.IsValid ()) {
			Count index = 0;
			OPDictIteratorRef iter;
			
			while (this->iterators->Get (index++, iter)) {
				if (iter.IsValid ()) {
					iter->NotifyRemove (elem);
				}
			}
		}
		
		next = elem->next;
		DELETE (elem);
		elem = next;
	}
	
	this->head = 0;
	this->tail = 0;
	
	this->num_element = 0;
}



/*
 *	Remove an object specified by its value.
 */

Bool
OPDict::RemoveValue (OPObjectIn value, Count nth)
{
	OP_METHOD (RemoveValue, 2);
	OP_ARG (1, OPObjectIn, value);
	OP_ARG (2, Count, nth);
	
	Element* elem = this->head;
	
	while (elem) {
		
		if ( (elem->value == value)
		  && (nth-- == 0) ) {
			
			if (this->iterators.IsValid ()) {
				Count index = 0;
				OPDictIteratorRef iter;
				
				while (this->iterators->Get (index++, iter)) {
					if (iter.IsValid ()) {
						iter->NotifyRemove (elem);
					}
				}
			}
			
			(elem->next ? elem->next->prev : this->tail) = elem->prev;
			(elem->prev ? elem->prev->next : this->head) = elem->next;
			
			this->num_element--;
			
			DELETE (elem);
			
		    return TRUE;
		}
		
		elem = elem->next;
	}
	
	return FALSE;
}



/*
 *	Find the key/value pair based on an index in the
 *	dictionary.
 */

Bool
OPDict::Get (Count pos, OPObjectOut key, OPObjectOut value)
{
	OP_METHOD (Get, 3);
	OP_ARG (1, Count, pos);
	OP_ARG (2, OPObjectOut, key);
	OP_ARG (3, OPObjectOut, value);
	
	if (pos >= this->num_element) {
		return FALSE;
	}
	
	Element* elem = this->head;
	
	while (pos--) {
		elem = elem->next;
	}
	
	key   = elem->key;
	value = elem->value;
	
	return TRUE;
}



/*
 *	Find the value of the nth corresponding key. Return
 *	FALSE on failure.
 */

Bool
OPDict::GetFromKey (OPObjectIn key, OPObjectOut value, Count nth)
{
	OP_METHOD (GetFromKey, 3);
	OP_ARG (1, OPObjectIn, key);
	OP_ARG (2, OPObjectOut, value);
	OP_ARG (3, Count, nth);
	
	Element* elem = this->head;
	
	if (key.IsValid () == FALSE) {
		return FALSE;
	}
	
	while (elem) {
		
		if ( (elem->key == key)
		  || ( (elem->key.IsValid ())
		    && (elem->key->CompareWith (key) == OP_COMP_THIS_SAME) ) ) {
			
			if (nth-- == 0) {
				value = elem->value;
				return TRUE;
			}
		}
		
		elem = elem->next;
	}
	
	return FALSE;
}



/*
 *	Find the key of the nth corresponding value. Return
 *	FALSE on failure.
 */

Bool
OPDict::GetFromValue (OPObjectIn value, OPObjectOut key, Count nth)
{
	OP_METHOD (GetFromValue, 3);
	OP_ARG (1, OPObjectIn, value);
	OP_ARG (2, OPObjectOut, key);
	OP_ARG (3, Count, nth);
	
	Element* elem = this->head;
	
	while (elem) {
		
		if ( (elem->value == value)
		  && (nth-- == 0) ) {
			
			key = elem->key;
			return TRUE;
		}
		
		elem = elem->next;
	}
	
	return FALSE;
}


/*
 *	Search for the specified key. If it exists, just return
 *	TRUE.
 */

Bool
OPDict::LookFor (OPObjectIn key)
{
	OP_METHOD (LookFor, 1);
	OP_ARG (1, OPObjectIn, key);
	
	Element* elem = this->head;
	
	while (elem) {
		
		if (elem->key == key) {
			return TRUE;
		}
		
		elem = elem->next;
	}
	
	return FALSE;
}
