/*
 *	rtinfo.cxx
 *
 *	This file implements the low-level run-time collector class
 *	used during start-up.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include <OPaC/object.h>

OP_IMPLEMENTATION_FILE;

/****************************************************************/

OPaC_RuntimeInfo* OPaC_RuntimeInfo::last = 0;

/****************************************************************/

/*
 *	Each class which has been declared using an OP_?_CLASS
 *	macro will instanciate an instance of this class.
 *
 *	Therefore, this method will be called for each class
 *	while staring up the OPaC class library.
 */

OPaC_RuntimeInfo::OPaC_RuntimeInfo (OPRuntimeInOut runtime,
									OPaC_ReturnRuntimeFunc runtime_func,
					  				const LatinChar* class_name,
					  				const LatinChar* file_name,
					  				Card32 instance_size,
					  				OPaC_ConstrFunc constr_func,
					  				OPaC_ConstrFunc iconstr_func,
					  				Bool is_abstract,
					  				Bool is_special)
{
	//	It will be ClassMan::Init's responsibility to create the 'OPRuntime'
	//	instances, once every run-time class has been hooked in this list :
	
	this->prev = this->last;
	this->last = this;
	
	this->runtime = & runtime;					//	remember pointer to OPRuntimeRef in class body
	
	this->runtime_func     = runtime_func;		//	function returning super-class's OPRuntimeRef
	this->constr_func      = constr_func;		//	function constructing an instance of this class
	this->init_constr_func = iconstr_func;		//	function constructing a dummy instance (booting)
	
	this->class_name = class_name;				//	class name
	this->file_name  = file_name;				//	implementation file name
	
	this->instance_size = instance_size;		//	size of an instance of this class
	
	this->is_abstract = is_abstract;			//	TRUE => may not allocate instances of this class
	this->is_special  = is_special;				//	TRUE => allocation done by reference class only
}

/****************************************************************/
