/*
 *	stepview.cxx
 *
 *	The Step-by-Step View holds many views of which only one can be
 *	visible at a given time. It can be used to implement wizards.
 *
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include <OPaC/stepview.h>
#include <OPaC/window.h>

/****************************************************************/

OP_IMPLEMENTATION_FILE;
OP_IMPLEMENT_CLASS (OPStepView);

/****************************************************************/

OPStepView::OPStepView ()
{
	OP_METHOD (OPStepView, 0);
	
	this->active_index = 0;
}

OPStepView::~OPStepView ()
{
	OP_METHOD (~OPStepView, 0);
}



/*
 *	Save the view to the storage.
 */

void
OPStepView::StoreInstance (OPStorageIn storage)
{
	OP_METHOD (StoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	
	super::StoreInstance (storage);
	
	static OPStorageSync info ("<OPStepView>");
	
	storage << info;
	storage << OPStorageSync::begin;
	storage << OPStorageSync::end;
}



/*
 *	Restore the view from the storage.
 */

void
OPStepView::RestoreInstance (OPStorageIn storage)
{
	OP_METHOD (RestoreInstance, 1);
	OP_ARG (1, OPStorageIn, storage);
	
	super::RestoreInstance (storage);
	
	storage >> OPStorageSync::type;
	storage >> OPStorageSync::begin;
	storage >> OPStorageSync::end;
}
