/*
 *	gfxfontcache.h
 *
 *	The Font Cache provides the necessary support to use the FreeType
 *	engine to draw glyphs on a GfxSurface.
 *
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#ifndef _GFX_GFXFONTCACHE_H_
#define	_GFX_GFXFONTCACHE_H_

#include "gfxlibrary.h"
#include "gfxsurface.h"

/****************************************************************/

/*
 *	The font cache provides the basic drawing methods used to
 *	display True Type fonts (based on the FreeType library).
 */	

class GFX_PUBLIC GfxFontCache
{
public:
	struct GFX_PUBLIC GlyphGeom
	{
		Int16			x_min, x_max;
		Int16			y_min, y_max;
		Int16			x_bearing;
		Int16			y_bearing;
		Int16			x_advance;
		Int16			y_advance;
	};
protected:
	struct FreeType;
	
	struct GFX_PUBLIC GlyphInfo
	{
		GlyphInfo*		next;				//	next glyph with same hash value
		Card16			x, y, dx, dy;		//	box in the band
		Card16			unicode;			//	code of this glyph
		Card16			band_id;			//	band index (1..n)
		Card16			kern_id;			//	kern pair ID (offset into table)
		Card16			liga_id;			//	ligature ID (offset into table)
		GlyphGeom		geom;
		
		GlyphInfo ();
		~ GlyphInfo ();
	};
	
protected:
	GfxSurface**		bands;				//	array of pointers to bands
	Card32				num_bands;			//	number of band entries in array
	
	Card16				band_id;			//	current band ID (insertion point)
	Card16				band_ox;			//	offset in the band (insertion point)
	Card16				band_dx;			//	total width of band
	Card16				band_dy;			//	total height of band
	
	Int16				typo_ascender;		//	typographic ascender
	Int16				typo_descender;		//	typographic descender
	Int16				typo_linegap;		//	typographic line gap
	
	GlyphInfo			glyphs[256];		//	first level entry into "hash" table
	FreeType*			font;				//	description of FreeType / TrueType font
	int					size;				//	font size
	
protected:
	void CharToUnicode ();
	GlyphInfo* NewGlyphInfo (Card16 unicode);
	GlyphInfo* FindGlyphInfo (Card16 unicode);
	
	void NewBand ();
	void GrowBands ();
	void NewGlyphInBand (Card16 dx, Card16& x, Card16& band_id);
	
	void FillGlyphInfo (GlyphInfo* info);
	
public:
	GfxFontCache ();
	virtual ~ GfxFontCache ();
	
	Bool OpenFace (const char* ttf_name, int size);
	void CloseFace ();
	
	void AccessGlyph (Card16 unicode, GfxSurface*& surface, Card16& x, Card16& y, GlyphGeom& geom);
	void GetGlyphGeometry (Card16 unicode, GlyphGeom& geom);
	void DrawGlyph (Card16 unicode, Card32 x, Card32 y, Card32 pixel, GfxSurface* surface);
	
	Int16 TypoAscender () const		{ return this->typo_ascender; }
	Int16 TypoDescender () const	{ return - this->typo_descender; }
	Int16 TypoLineGap () const		{ return this->typo_linegap; }
};

/****************************************************************/

#endif /* _GFX_GFXFONTCACHE_H_ */
