/*
 *	main.cxx
 *
 *	This application is currently a stand-alone interface builder
 *	which will gradually be integrated into the OPaC class library.
 *
 *	(C) Copyright 1994-1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include "main.h"
#include "appmacro.h"


#include <OPaC/dialog.h>
#include <OPaC/event.h>
#include <OPaC/ib.h>
#include <OPaC/new.h>
#include <OPaC/window.h>

#include <OPaC/notmacro.h>
#include <OPaC/pubmacro.h>
#include <OPaC/targetmacro.h>
#include <OPaC/values.h>

/****************************************************************/

APP_IMPLEMENTATION_FILE;
APP_IMPLEMENT_CLASS (App);

/****************************************************************/

OP_TARGET_ACTOR	(App, Terminate);
OP_TARGET_ACTOR	(App, MinimiseAll);

OP_ACCESSOR (App, X, Real, x);
OP_ACCESSOR (App, Y, Real, y);

OP_PUBLISH_SLOT_READ (App, z, ReturnZ, "z = x * y", Real);

OP_NOTIFY_CHANGE (App, z, Changed);

/****************************************************************/

App::App ()
{
	this->x = 0;
	this->y = 0;
}

App::~App ()
{
	this->dialog = OPDialogNull;
}


/*
 *	Initialise the application.
 */

void
App::Initialise (Bool enable_build)
{
	super::Initialise (enable_build);
	
	//	Load the interface for the dialog and open the corresponding window.
	
	this->dialog = OP_NEW (OPDialog);
	this->dialog->Interaction ("OPaC Demonstration");
	this->dialog->ReturnWindow ()->ShowCentered ();
}

/****************************************************************/

/*
 *	Terminate the application. This must be connected or
 *	there will be no means of quitting.
 */

void
App::Terminate ()
{
	OPEvent::PostQuitEvent ();
}

void
App::MinimiseAll ()
{
	int num   = this->window_num;
	int index = 0;

	while (num--) {
		OPWindowRef window;
		if (this->window_list->Get (index++, window)) {
			window->ShowReduced ();
		}
	}
}

/****************************************************************/

Real
App::ReturnZ () const
{
	return this->x * this->y;
}

