/*
 *	buildnew.cxx
 *
 *	Support class used to create a new instance of an object.
 *
 *	(C) Copyright 1998, Pierre ARNAUD, OPaC bright ideas
 *		CH-1437 SUSCEVAZ, Switzerland. All rights reserved.
 */

/******************************************************************************

  This file is part of the OPaC class library ("OPaC").

  OPaC is distributed with NO WARRANTY OF ANY KIND.  No author or distributor
  accepts any responsibility for the consequences of using it, or for whether
  it serves any particular purpose or works at all, unless he or she says so
  in writing. Refer to the OPaC Free Public License (the "License") for full
  details.

  Every copy of OPaC must include a copy of the License, normally in a plain
  ASCII text file named PUBLIC. The License grants you the right to copy,
  modify and redistribute OPaC, but only under certain conditions described
  in the License. The copyright notice and this notice must be preserved on
  all copies.

******************************************************************************/

#include "buildnew.h"
#include "buildedit.h"

#include <OPaC/array.h>
#include <OPaC/boxview.h>
#include <OPaC/button.h>
#include <OPaC/cellgrid.h>
#include <OPaC/container.h>
#include <OPaC/dbscrlist.h>
#include <OPaC/dragpal.h>
#include <OPaC/icon.h>
#include <OPaC/icostyle.h>
#include <OPaC/message.h>
#include <OPaC/new.h>
#include <OPaC/opacirt.h>
#include <OPaC/selgrid.h>
#include <OPaC/stepview.h>
#include <OPaC/textstyle.h>
#include <OPaC/texttag.h>
#include <OPaC/typeinfo.h>
#include <OPaC/values.h>
#include <OPaC/window.h>

#include <OPaC/notmacro.h>
#include <OPaC/targetmacro.h>

/****************************************************************/

OP_IMPLEMENTATION_FILE;
APP_IMPLEMENT_CLASS (OPBuildNew);

OP_TARGET_ACTOR (OPBuildNew, OnPrevStep);
OP_TARGET_ACTOR (OPBuildNew, OnNextStep);
OP_TARGET_ACTOR (OPBuildNew, OnCancel);

OP_NOTIFY_EVENT (OPBuildNew, InitFinished);

/****************************************************************/

OPBuildNew::OPBuildNew ()
{
	OP_METHOD (OPBuildNew, 0);
	
	this->step_2_progress = 0;
	this->step_2_total    = 0;
	
	this->label_finish = "Finish";
	this->hide_cancel  = FALSE;
	this->auto_finish  = TRUE;
}

OPBuildNew::~OPBuildNew ()
{
}

/****************************************************************/

void
OPBuildNew::Initialise (OPRuntimeIn base)
{
	this->window = OP_NEW (OPWindow);
	this->base_runtime = base;
	
	this->window->Initialise (20, 20, 320, 270, "Interface Builder - New Object");
	this->window->SetStyle (OP_WIN_TITLE | OP_WIN_MOVE_OK);
	this->window->SetFlagOkDrop (FALSE);
	
	OPListRef list = OP_NEW (OPList);
	OPBoxViewRef boxview;
	
	list->SetInsertMode (FALSE);
	
	boxview = OP_NEW (OPBoxView);
	boxview->SetEnableTitle (FALSE);
	boxview->SetEnableFrame (FALSE);
	this->SetupViewList (boxview);
	list->Insert (boxview);
	
	boxview = OP_NEW (OPBoxView);
	boxview->SetEnableTitle (FALSE);
	boxview->SetEnableFrame (FALSE);
	this->SetupViewArgs (boxview);
	list->Insert (boxview);
	
	boxview = OP_NEW (OPBoxView);
	boxview->SetEnableTitle (FALSE);
	boxview->SetEnableFrame (FALSE);
	this->SetupViewFinish (boxview);
	list->Insert (boxview);
	
	this->step = OP_NEW (OPStepView);
	this->step->SetParent (this->window);
	this->step->SetOrigin (10, 10);
	this->step->SetViewList (list);
	this->step->AdjustOverallSize ();
	
	OPCoord dx;
	OPCoord dy;
	
	this->step->GetSize (dx, dy);
	
	if (dx < 300) dx = 300;
	
	this->window->SetSize (10+dx+10, 10+dy+10+24+10);
	
	this->button_prev = OP_NEW (OPButton);
	this->button_prev->Initialise (10, 10+dy+10, 80, 24, "Previous", this->window);
	this->button_prev->AttachObserver (this, "OnClick", "OnPrevStep");
	
	this->button_next = OP_NEW (OPButton);
	this->button_next->Initialise (10+80, 10+dy+10, 80, 24, "Next", this->window);
	this->button_next->AttachObserver (this, "OnClick", "OnNextStep");
	
	this->button_cancel = OP_NEW (OPButton);
	this->button_cancel->Initialise (10+dx-80, 10+dy+10, 80, 24, "Cancel", this->window);
	this->button_cancel->AttachObserver (this, "OnClick", "OnCancel");
	
	this->window->Open ();
	this->Step1 ();
	this->window->ShowNormal ();
}

void
OPBuildNew::DefineFinishButton (OPStringIn label, Bool auto_finish, Bool hide_cancel)
{
	this->auto_finish  = auto_finish;
	this->hide_cancel  = hide_cancel;
	
	if (label.IsValid ()) {
		this->label_finish = label->ReturnString ();
	} else {
		this->label_finish = "Close";
	}
}

void
OPBuildNew::SetupViewList (OPBoxViewInOut boxview)
{
	Count max = OPClassMan::TotalCount ();
	Count index;
	
	OPDBScrollListRef list = OP_NEW (OPDBScrollList);
	
	list->SetParent (boxview);
	list->SetOrigin (0, 0);
	list->SetSize (180, 170);
	list->SetupDataBase ("Classes");
	
	for (index = 0; index < max; index++) {
		
		OPRuntimeRef runtime;
		OPClassMan::GetClass (index, runtime);
		
		if (runtime->IsKindOf (this->base_runtime)) {
			OPAtomRef name = runtime->ReturnClassName ();
			list->Insert (name, name, runtime, name);
		}
	}
	
	list->Update ();
	
	boxview->SetDesiredMinSize (180, 170);
	
	this->step_1_list = list;
}


void
OPBuildNew::SetupViewArgs (OPBoxViewInOut boxview)
{
	boxview->SetDesiredMinSize (320, 200);
	boxview->SetSize (320, 200);
}

void
OPBuildNew::SetupViewFinish (OPBoxViewInOut boxview)
{
	boxview->SetDesiredMinSize (320, 200);
	boxview->SetSize (320, 200);
}

/****************************************************************/

void
OPBuildNew::EmptyView (OPViewIn view)
{
	if (view.IsValid ()) {
		OPViewRef child;
		
		while (view->GetFirstChild (child)) {
			child->SetParent (OPViewNull);
		}
	}
}

/****************************************************************/

/*
 *	The first step is easy: just show the list of runtime
 *	classes available for instanciation.
 */

Bool
OPBuildNew::Step1 (Bool back)
{
	this->button_prev->SetEnable (FALSE);
	this->button_next->SetEnable (TRUE);
	this->step->ActivateView (1);
	this->window->Display ();
	
	return TRUE;
}

/*
 *	The second step requires some work :
 *
 *	- Allocate the user object
 *	- Setup the values which will be needed for the object
 *	  initialisation.
 */

Bool
OPBuildNew::Step2 (Bool back)
{
	this->button_prev->SetEnable (TRUE);
	this->button_next->SetEnable (TRUE);
	this->button_next->SetTitle ("Next");
	this->button_cancel->SetParent (this->window);
	
	if (back) {
		
		//	If no initialisation value is needed, skip the second step and move
		//	immediately to the first one.
		
		if (this->step_2_total == 0) {
			return FALSE;
		}
		
		this->step_2_progress = this->step_2_total-1;
		this->Step2SubStep ();
		
		return TRUE;
	}
	
	if (this->step_1_list->GetSelectedData (this->user_runtime)) {
		
		Bool already_ok = this->user_object.IsValid () && this->user_object->IsA (this->user_runtime);
		
		OPBoxViewRef boxview;
		const OPaC_Target* init_target = this->user_runtime->FindMethod ("Initialise", 0);
		
		this->step->GetNthView (2, boxview);
		this->EmptyView (boxview);
		
		//	Allocate the object.
		
		if (already_ok == FALSE) {
			this->user_object = OP_DYNAMIC_NEW (this->user_runtime, OPObject);
			this->step_2_progress = 0;
			this->step_2_total    = 0;
		}
		
		//	If no target initialisation method exists, simply skip the interactive
		//	part of this dialog.
		
		if (init_target == 0) {
			return FALSE;
		}
		
		if (already_ok) {
			this->Step2SubStep ();
			return TRUE;
		}
		
		const OPaC_TargetMethod* method = init_target->ReturnTargetMethod ();
		const OPaC_ParamDesc* desc = & method->param[0];
		
		Count num   = method->num_param;
		Count index = 0;
		
		this->user_values    = OP_NEW (OPArray);
		this->user_initviews = OP_NEW (OPArray);
		
		this->user_values->SetSize (num);
		this->user_initviews->SetSize (num);
		
		this->step_2_total = num;
		
		for (index = 0; index < num; index++) {
			
			OPViewRef     view    = OPViewNull;
			OPTextTagRef  caption = OP_NEW (OPTextTag);
			OPBoxViewRef  bview   = OP_NEW (OPBoxView);
			OPTypeInfoRef type    = OP_NEW (OPTypeInfo);
			OPStringRef   text    = desc[index].desc;
			
			this->user_values[index] = OP_NEW (OPValue);
			
			type->Initialise (desc[index].type_def);
			text->AppendString (" :  ");
			
			OPCoord mx = 0;
			OPCoord my = 0;
			
			caption->SetTitle (text);
			caption->SetMode (OP_TEXT_HPOS_LEFT | OP_TEXT_VPOS_CENTER);
			caption->GetMinSize (mx, my);
			
			OPSimpleType what = OPBuildEdit::FindSimpleType (this->user_values[index], type, "object");
			
			if (what == OP_TYPE_OBJECT) {
				OPBuildEdit::EditAnyProperty (0, this->user_values[index], type, "object", view);
			} else {
				OPBuildEdit::EditAnyProperty (0, this->user_values[index], type, "text", view);
			}
			
			OPCoord dx = 0;
			OPCoord dy = 24;
			
			if (view.IsValid ()) {
				view->SetParent (bview);
				view->SetOrigin (0, my+10);
				view->GetSize (dx, dy);
			}
			
			if (mx > dx) dx = mx;
			
			caption->SetParent (bview);
			caption->SetOrigin (0, 0);
			caption->SetSize (mx, my);
			
			bview->SetEnableFrame (FALSE);
			bview->SetEnableTitle (FALSE);
			bview->SetDesiredMinSize (dx, my+10+dy);
			bview->SetSize (dx, my+10+dy);
			
			this->user_initviews[index] = bview;
		}
		
		this->Step2SubStep ();
		return TRUE;
	}
	
	return FALSE;
}

/*
 *	The second step consists of several initialisation steps,
 *	one for each argument to the initialisation method.
 */

void
OPBuildNew::Step2SubStep ()
{
	OPBoxViewRef boxview;
	this->step->GetNthView (2, boxview);
	this->EmptyView (boxview);
	
	this->user_initviews[this->step_2_progress]->SetParent (boxview);
	this->user_initviews[this->step_2_progress]->SetOrigin (10, 10);
	
	this->step->ActivateView (2);
	this->window->Display ();
}


/*
 *	The third step allocates and initialises the object. It
 *	will be displayed, if possible, in a drag palette.
 */

Bool
OPBuildNew::Step3 (Bool back)
{
	this->button_next->SetTitle (this->label_finish);
	
	if (this->hide_cancel) {
		this->button_cancel->SetParent (OPViewNull);
	}
	
	OPBoxViewRef boxview;
	this->step->GetNthView (3, boxview);
	this->EmptyView (boxview);
	
	this->user_object = OP_DYNAMIC_NEW (this->user_runtime, OPObject);
	
	Bool is_window = this->user_object->IsKindOf (OP_RUNTIME (OPWindow));
	Bool is_view   = this->user_object->IsKindOf (OP_RUNTIME (OPView));
	
	if (is_view) {
		OPViewRef view = this->user_object;
		view->SetSize (100, 100);
	}
	
	if (this->step_2_total) {
		
		OPMessageRef msg = OP_NEW (OPMessage);
		
		msg->SetMethod ("Initialise");
		msg->Clear ();
		
		for (Count index = 0; index < this->step_2_total; index++) {
			
			OPValueRef   value  = this->user_values[index];
			OPObjectRef  object = value;
			OPSimpleType type   = value->ReturnType ();
			
			while ( (type == OP_TYPE_OBJECT)
				 || (type == OP_TYPE_TEXT)
				 || (type == OP_TYPE_ARRAY) ) {
				
				object = value->ReturnObject ();
				
				if (object->IsKindOf (OP_RUNTIME (OPValue)) == FALSE) break;
				
				value = object;
				type  = value->ReturnType ();
			}
			
			msg << object;
		}
		
		msg->Send (this->user_object);
	}
	
	OPTextTagRef caption;
	OPStringRef text;
	
	text = "Initialisation of class ";
	text->AppendString (this->user_runtime->ReturnClassName ());
	text->AppendString (" completed.");
	
	caption = OP_NEW (OPTextTag);
	caption->Initialise (10, 10, 300, 24, text, boxview);
	caption->SetMode (OP_TEXT_HPOS_LEFT | OP_TEXT_VPOS_CENTER);
	
	if (!is_window) {
		
		OPDragPaletteRef drag = OP_NEW (OPDragPalette);
		OPViewRef view;
		
		if (is_view) {
			view = this->user_object;
		} else {
			OPIconRef icon = OP_NEW (OPIcon);
			icon->Initialise (OP_ICON_CUBE_NORMAL);
			icon->SetDefaultSize ();
			view = icon;
			view->SetAttribute ("embedded_object_for_drag", this->user_object);
			view->SetAttribute ("embedded_object_for_connect", this->user_object);
		}
		
		OPCoord dx;
		OPCoord dy;
		OPCoord bdx;
		OPCoord bdy;
		
		boxview->GetSize (bdx, bdy);
		view->GetSize (dx, dy);
		
		if ( (dx == 0)
		  || (dy == 0) ) {
			view->GetMinSize (dx, dy);
		}
		
		drag->SetParent (boxview);
		drag->SetOrigin ((bdx-dx)/2, 10+(bdy-44-dy)/2);
		drag->SetSize (dx, dy);
		view->SetParent (drag);
		view->SetBox (drag->ReturnBox ());
	}
	
	this->step->ActivateView (3);
	this->window->Display ();
	
	return TRUE;
}

Bool
OPBuildNew::Step4 (Bool back)
{
	this->Report (notify_InitFinished);
	
	if (this->auto_finish) {
		
		OPBuildNewRef self = this;
		
		this->window->Delete ();			this->window = 0;
		
		this->button_prev->Delete ();		this->button_prev = 0;
		this->button_next->Delete ();		this->button_next = 0;
		this->button_cancel->Delete ();		this->button_cancel = 0;
		
		this->user_object = 0;
		this->user_values = 0;
		this->user_initviews = 0;
		this->step_1_list = 0;
		this->step_2_progress = 0;
		this->step_2_total = 0;

		this->Delete ();
	}
	
	return TRUE;
}

/****************************************************************/

void
OPBuildNew::OnNextStep ()
{
	Count step = this->step->ReturnActiveView ();
	
	if ( (step == 2)
	  && (this->step_2_progress < this->step_2_total-1) ) {
		
		this->step_2_progress++;
		this->Step2SubStep ();
		return;
	}
	
	switch (step) {
		case 1:
			if (this->Step2 (FALSE)) break;
		case 2:
			if (this->Step3 (FALSE)) break;
		case 3:
			if (this->Step4 (FALSE)) break;
			break;
	}
}

void
OPBuildNew::OnPrevStep ()
{
	Count step = this->step->ReturnActiveView ();
	
	if ( (step == 2)
	  && (this->step_2_progress > 0) ) {
		
		this->step_2_progress--;
		this->Step2SubStep ();
		return;
	}
	
	switch (step) {
		case 3:
			if (this->Step2 (TRUE)) break;
		case 2:
			if (this->Step1 (TRUE)) break;
			break;
	}
}

void
OPBuildNew::OnCancel ()
{
	OPObjectRef self = this;
	this->window->Delete ();
	this->button_prev->Delete ();
	this->button_next->Delete ();
	this->button_cancel->Delete ();
}

/****************************************************************/

Bool
OPBuildNew::GetObject (OPObjectOut object) const
{
	object = this->user_object;
	return object.IsValid ();
}
